# coding: utf-8
# cython: boundscheck=False
# cython: nonecheck=False
# cython: cdivision=True
# cython: wraparound=False
# cython: profile=False

""" Built-in potentials implemented in Cython """

from __future__ import division, print_function

# Standard library
from collections import OrderedDict

# Third-party
from astropy.extern import six
from astropy.utils import InheritDocstrings
from astropy.constants import G
import astropy.units as u
import numpy as np
cimport numpy as np
np.import_array()

# Project
from ..core import CompositePotential
from ..cpotential import CPotentialBase
from ..cpotential cimport CPotential, CPotentialWrapper
from ..cpotential cimport densityfunc, energyfunc, gradientfunc, hessianfunc
from ...frame.cframe cimport CFrameWrapper
from ....units import dimensionless, DimensionlessUnitSystem

cdef extern from "potential/builtin/builtin_potentials.h":
    double henon_heiles_value(double t, double *pars, double *q, int n_dim) nogil
    void henon_heiles_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil

    double kepler_value(double t, double *pars, double *q, int n_dim) nogil
    void kepler_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    void kepler_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil

    double isochrone_value(double t, double *pars, double *q, int n_dim) nogil
    void isochrone_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double isochrone_density(double t, double *pars, double *q, int n_dim) nogil
    void isochrone_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil

    double hernquist_value(double t, double *pars, double *q, int n_dim) nogil
    void hernquist_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double hernquist_density(double t, double *pars, double *q, int n_dim) nogil
    void hernquist_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil

    double plummer_value(double t, double *pars, double *q, int n_dim) nogil
    void plummer_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double plummer_density(double t, double *pars, double *q, int n_dim) nogil
    void plummer_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil

    double jaffe_value(double t, double *pars, double *q, int n_dim) nogil
    void jaffe_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double jaffe_density(double t, double *pars, double *q, int n_dim) nogil

    double stone_value(double t, double *pars, double *q, int n_dim) nogil
    void stone_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double stone_density(double t, double *pars, double *q, int n_dim) nogil

    double sphericalnfw_value(double t, double *pars, double *q, int n_dim) nogil
    void sphericalnfw_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double sphericalnfw_density(double t, double *pars, double *q, int n_dim) nogil
    void sphericalnfw_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil

    double flattenednfw_value(double t, double *pars, double *q, int n_dim) nogil
    void flattenednfw_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil

    double triaxialnfw_value(double t, double *pars, double *q, int n_dim) nogil
    void triaxialnfw_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil

    double satoh_value(double t, double *pars, double *q, int n_dim) nogil
    void satoh_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double satoh_density(double t, double *pars, double *q, int n_dim) nogil

    double miyamotonagai_value(double t, double *pars, double *q, int n_dim) nogil
    void miyamotonagai_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    void miyamotonagai_hessian(double t, double *pars, double *q, int n_dim, double *hess) nogil
    double miyamotonagai_density(double t, double *pars, double *q, int n_dim) nogil

    double leesuto_value(double t, double *pars, double *q, int n_dim) nogil
    void leesuto_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil
    double leesuto_density(double t, double *pars, double *q, int n_dim) nogil

    double logarithmic_value(double t, double *pars, double *q, int n_dim) nogil
    void logarithmic_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil

    double longmuralibar_value(double t, double *pars, double *q, int n_dim) nogil
    void longmuralibar_gradient(double t, double *pars, double *q, int n_dim, double *grad) nogil

__all__ = ['HenonHeilesPotential', # Misc. potentials
           'KeplerPotential', 'HernquistPotential', 'IsochronePotential', 'PlummerPotential',
           'JaffePotential', 'StonePotential', # Spherical models
           'SatohPotential', 'MiyamotoNagaiPotential', # Disk models
           'NFWPotential', 'LeeSutoTriaxialNFWPotential', 'LogarithmicPotential',
           'LongMuraliBarPotential', # Triaxial models
           'SphericalNFWPotential', 'FlattenedNFWPotential' # Deprecated
           ]

# ============================================================================

cdef class HenonHeilesWrapper(CPotentialWrapper):

    def __init__(self, G, _, q0):
        self.init([G], np.ascontiguousarray(q0), n_dim=2)
        self.cpotential.value[0] = <energyfunc>(henon_heiles_value)
        self.cpotential.gradient[0] = <gradientfunc>(henon_heiles_gradient)

class HenonHeilesPotential(CPotentialBase):
    r"""
    HenonHeilesPotential(units=None, origin=None)

    The Hénon-Heiles potential.

    .. math::

        \Phi(x,y) = \frac{1}{2}(x^2 + y^2 + 2x^2 y - \frac{2}{3}y^3)

    Parameters
    ----------
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, units=None, origin=None):
        parameters = OrderedDict()
        super(HenonHeilesPotential, self).__init__(parameters=parameters,
                                                   parameter_physical_types=dict(),
                                                   ndim=2,
                                                   units=units,
                                                   origin=origin)


# ============================================================================
# Spherical models
#
cdef class KeplerWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(kepler_value)
        self.cpotential.gradient[0] = <gradientfunc>(kepler_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(kepler_hessian)

class KeplerPotential(CPotentialBase):
    r"""
    KeplerPotential(m, units=None, origin=None)

    The Kepler potential for a point mass.

    .. math::

        \Phi(r) = -\frac{Gm}{r}

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Point mass value.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        super(KeplerPotential, self).__init__(parameters=parameters,
                                              parameter_physical_types=ptypes,
                                              units=units,
                                              origin=origin)


cdef class IsochroneWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(isochrone_value)
        self.cpotential.density[0] = <densityfunc>(isochrone_density)
        self.cpotential.gradient[0] = <gradientfunc>(isochrone_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(isochrone_hessian)

class IsochronePotential(CPotentialBase):
    r"""
    IsochronePotential(m, b, units=None, origin=None)

    The Isochrone potential.

    .. math::

        \Phi = -\frac{GM}{\sqrt{r^2+b^2} + b}

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Mass.
    b : :class:`~astropy.units.Quantity`, numeric [length]
        Core concentration.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, b, units=None, origin=None):
        ptypes = OrderedDict()
        parameters = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['b'] = b
        ptypes['b'] = 'length'

        super(IsochronePotential, self).__init__(parameters=parameters,
                                                 parameter_physical_types=ptypes,
                                                 units=units,
                                                 origin=origin)

    def action_angle(self, w):
        """
        Transform the input cartesian position and velocity to action-angle
        coordinates the Isochrone potential. See Section 3.5.2 in
        Binney & Tremaine (2008), and be aware of the errata entry for
        Eq. 3.225.

        This transformation is analytic and can be used as a "toy potential"
        in the Sanders & Binney 2014 formalism for computing action-angle
        coordinates in _any_ potential.

        Adapted from Jason Sanders' code
        `here <https://github.com/jlsanders/genfunc>`_.

        Parameters
        ----------
        w : :class:`gala.dynamics.PhaseSpacePosition`, :class:`gala.dynamics.Orbit`
            The positions or orbit to compute the actions, angles, and frequencies at.
        """
        from ....dynamics.analyticactionangle import isochrone_to_aa
        return isochrone_to_aa(w, self)

    # def phase_space(self, actions, angles):
    #     """
    #     Transform the input actions and angles to ordinary phase space (position
    #     and velocity) in cartesian coordinates. See Section 3.5.2 in
    #     Binney & Tremaine (2008), and be aware of the errata entry for
    #     Eq. 3.225.

    #     Parameters
    #     ----------
    #     actions : array_like
    #     angles : array_like
    #     """
    #     from ...dynamics.analyticactionangle import isochrone_aa_to_xv
    #     return isochrone_aa_to_xv(actions, angles, self)


cdef class HernquistWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(hernquist_value)
        self.cpotential.density[0] = <densityfunc>(hernquist_density)
        self.cpotential.gradient[0] = <gradientfunc>(hernquist_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(hernquist_hessian)

class HernquistPotential(CPotentialBase):
    r"""
    HernquistPotential(m, c, units=None, origin=None)

    Hernquist potential for a spheroid.

    .. math::

        \Phi(r) = -\frac{G M}{r + c}

    See: http://adsabs.harvard.edu/abs/1990ApJ...356..359H

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Mass.
    c : :class:`~astropy.units.Quantity`, numeric [length]
        Core concentration.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, c, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['c'] = c
        ptypes['c'] = 'length'

        super(HernquistPotential, self).__init__(parameters=parameters,
                                                 parameter_physical_types=ptypes,
                                                 units=units,
                                                 origin=origin)


cdef class PlummerWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(plummer_value)
        self.cpotential.density[0] = <densityfunc>(plummer_density)
        self.cpotential.gradient[0] = <gradientfunc>(plummer_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(plummer_hessian)

class PlummerPotential(CPotentialBase):
    r"""
    PlummerPotential(m, b, units=None, origin=None)

    Plummer potential for a spheroid.

    .. math::

        \Phi(r) = -\frac{G M}{\sqrt{r^2 + b^2}}

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
       Mass.
    b : :class:`~astropy.units.Quantity`, numeric [length]
        Core concentration.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, b, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['b'] = b
        ptypes['b'] = 'length'

        super(PlummerPotential, self).__init__(parameters=parameters,
                                               parameter_physical_types=ptypes,
                                               units=units,
                                               origin=origin)


cdef class JaffeWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(jaffe_value)
        self.cpotential.density[0] = <densityfunc>(jaffe_density)
        self.cpotential.gradient[0] = <gradientfunc>(jaffe_gradient)

class JaffePotential(CPotentialBase):
    r"""
    JaffePotential(m, c, units=None, origin=None)

    Jaffe potential for a spheroid.

    .. math::

        \Phi(r) = \frac{G M}{c} \ln(\frac{r}{r + c})

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Mass.
    c : :class:`~astropy.units.Quantity`, numeric [length]
        Core concentration.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, c, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['c'] = c
        ptypes['c'] = 'length'

        super(JaffePotential, self).__init__(parameters=parameters,
                                             parameter_physical_types=ptypes,
                                             units=units,
                                             origin=origin)


cdef class StoneWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(stone_value)
        self.cpotential.density[0] = <densityfunc>(stone_density)
        self.cpotential.gradient[0] = <gradientfunc>(stone_gradient)

class StonePotential(CPotentialBase):
    r"""
    StonePotential(m, r_c, r_h, units=None, origin=None)

    Stone potential from `Stone & Ostriker (2015) <http://dx.doi.org/10.1088/2041-8205/806/2/L28>`_.

    .. math::

        \Phi(r) = -\frac{2 G M}{\pi(r_h - r_c)}\left[ \frac{\arctan(r/r_h)}{r/r_h} - \frac{\arctan(r/r_c)}{r/r_c} + \frac{1}{2}\ln\left(\frac{r^2+r_h^2}{r^2+r_c^2}\right)\right]

    Parameters
    ----------
    m_tot : :class:`~astropy.units.Quantity`, numeric [mass]
        Total mass.
    r_c : :class:`~astropy.units.Quantity`, numeric [length]
        Core radius.
    r_h : :class:`~astropy.units.Quantity`, numeric [length]
        Halo radius.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, r_c, r_h, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['r_c'] = r_c
        ptypes['r_c'] = 'length'

        parameters['r_h'] = r_h
        ptypes['r_h'] = 'length'

        super(StonePotential, self).__init__(parameters=parameters,
                                             parameter_physical_types=ptypes,
                                             units=units,
                                             origin=origin)


# ============================================================================
# Flattened, axisymmetric models
#
cdef class SatohWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(satoh_value)
        self.cpotential.density[0] = <densityfunc>(satoh_density)
        self.cpotential.gradient[0] = <gradientfunc>(satoh_gradient)

class SatohPotential(CPotentialBase):
    r"""
    SatohPotential(m, a, b, units=None, origin=None)

    Satoh potential for a flattened mass distribution.

    .. math::

        \Phi(R,z) = -\frac{G M}{\sqrt{R^2 + z^2 + a(a + 2\sqrt{z^2 + b^2})}}

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Mass.
    a : :class:`~astropy.units.Quantity`, numeric [length]
        Scale length.
    b : :class:`~astropy.units.Quantity`, numeric [length]
        Scare height.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, a, b, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['a'] = a
        ptypes['a'] = 'length'

        parameters['b'] = b
        ptypes['b'] = 'length'

        super(SatohPotential, self).__init__(parameters=parameters,
                                             parameter_physical_types=ptypes,
                                             units=units,
                                             origin=origin)


cdef class MiyamotoNagaiWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(miyamotonagai_value)
        self.cpotential.density[0] = <densityfunc>(miyamotonagai_density)
        self.cpotential.gradient[0] = <gradientfunc>(miyamotonagai_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(miyamotonagai_hessian)

class MiyamotoNagaiPotential(CPotentialBase):
    r"""
    MiyamotoNagaiPotential(m, a, b, units=None, origin=None)

    Miyamoto-Nagai potential for a flattened mass distribution.

    .. math::

        \Phi(R,z) = -\frac{G M}{\sqrt{R^2 + (a + \sqrt{z^2 + b^2})^2}}

    See: http://adsabs.harvard.edu/abs/1975PASJ...27..533M

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Mass.
    a : :class:`~astropy.units.Quantity`, numeric [length]
        Scale length.
    b : :class:`~astropy.units.Quantity`, numeric [length]
        Scare height.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, a, b, units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['a'] = a
        ptypes['a'] = 'length'

        parameters['b'] = b
        ptypes['b'] = 'length'

        super(MiyamotoNagaiPotential, self).__init__(parameters=parameters,
                                                     parameter_physical_types=ptypes,
                                                     units=units,
                                                     origin=origin)


# ============================================================================
# Triaxial models
#

cdef class SphericalNFWWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(sphericalnfw_value)
        self.cpotential.density[0] = <densityfunc>(sphericalnfw_density)
        self.cpotential.gradient[0] = <gradientfunc>(sphericalnfw_gradient)
        self.cpotential.hessian[0] = <hessianfunc>(sphericalnfw_hessian)

cdef class FlattenedNFWWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(flattenednfw_value)
        self.cpotential.gradient[0] = <gradientfunc>(flattenednfw_gradient)

cdef class TriaxialNFWWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(triaxialnfw_value)
        self.cpotential.gradient[0] = <gradientfunc>(triaxialnfw_gradient)

class NFWPotential(CPotentialBase):
    r"""
    NFWPotential(m, r_s, a=1, b=1, c=1, units=None, origin=None)

    General Navarro-Frenk-White potential. Supports spherical, flattened, and
    triaxiality but the flattening is introduced into the potential, not the
    density, and can therefore lead to unphysical mass distributions. For a
    triaxial NFW potential that supports flattening in the density, see
    :class:`gala.potential.LeeSutoTriaxialNFWPotential`.

    .. math::

        \Phi(r) = -\frac{v_c^2}{\sqrt{\ln 2 - \frac{1}{2}}} \frac{\ln(1 + r/r_s)}{r/r_s}

    Parameters
    ----------
    m : :class:`~astropy.units.Quantity`, numeric [mass]
        Scale mass.
    r_s : :class:`~astropy.units.Quantity`, numeric [length]
        Scale radius.
    a : numeric
        Major axis scale.
    b : numeric
        Intermediate axis scale.
    c : numeric
        Minor axis scale.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m=None, r_s=None, a=1., b=1., c=1., v_c=None, units=None, origin=None):
        # TODO: v_c included in above for backwards-compatibility (and m, r_s default to None)

        if v_c is not None and m is None:
            import warnings
            warnings.warn("NFWPotential now expects a scale mass in the default initializer. "
                          "To initialize from a circular velocity, use the classmethod "
                          "from_circular_velocity() instead instead.", DeprecationWarning)

            parameters = OrderedDict()
            ptypes = OrderedDict()

            parameters['v_c'] = v_c
            ptypes['v_c'] = 'speed'

            parameters['r_s'] = r_s
            ptypes['r_s'] = 'length'

            # get appropriate units:
            parameters = CPotentialBase._prepare_parameters(parameters, ptypes, units)

            # r_ref = r_s for old parametrization
            m = NFWPotential._vc_rs_rref_to_m(parameters['v_c'], parameters['r_s'],
                                              parameters['r_s'])
            m = m.to(units['mass'])

        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        parameters['r_s'] = r_s
        ptypes['r_s'] = 'length'

        if np.allclose([a, b, c], 1.):
            NFWWrapper = SphericalNFWWrapper

        elif np.allclose([a, b], 1.):
            NFWWrapper = FlattenedNFWWrapper
            parameters['c'] = c

        else:
            NFWWrapper = TriaxialNFWWrapper
            parameters['a'] = a
            parameters['b'] = b
            parameters['c'] = c

        super(NFWPotential, self).__init__(parameters=parameters,
                                           parameter_physical_types=ptypes,
                                           units=units,
                                           Wrapper=NFWWrapper)

    @staticmethod
    def from_circular_velocity(v_c, r_s, a=1., b=1., c=1., r_ref=None, units=None, origin=None):
        r"""
        from_circular_velocity(v_c, r_s, a=1., b=1., c=1., r_ref=None, units=None, origin=None)

        Initialize an NFW potential from a circular velocity, scale radius, and
        reference radius for the circular velocity.

        For scale mass :math:`m_s`, scale radius :math:`r_s`, scaled
        reference radius :math:`u_{\rm ref} = r_{\rm ref}/r_s`:

        .. math::

            \frac{G\,m_s}{r_s} = \frac{v_c^2}{u_{\rm ref}} \,
                \left[\frac{u_{\rm ref}}{1+u_{\rm ref}} -
                \frac{\ln(1+u_{\rm ref})}{u_{\rm ref}^2} \right]^{-1}

        Parameters
        ----------
        v_c : :class:`~astropy.units.Quantity`, numeric [velocity]
            Circular velocity at the reference radius ``r_ref`` (see below).
        r_s : :class:`~astropy.units.Quantity`, numeric [length]
            Scale radius.
        a : numeric
            Major axis scale.
        b : numeric
            Intermediate axis scale.
        c : numeric
            Minor axis scale.
        r_ref : :class:`~astropy.units.Quantity`, numeric [length] (optional)
            Reference radius at which the circular velocity is given. By default,
            this is assumed to be the scale radius, ``r_s``.

        """

        if r_ref is None:
            r_ref = r_s

        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['v_c'] = v_c
        ptypes['v_c'] = 'speed'

        parameters['r_s'] = r_s
        ptypes['r_s'] = 'length'

        parameters['r_ref'] = r_ref
        ptypes['r_ref'] = 'length'

        # get appropriate units:
        parameters = CPotentialBase._prepare_parameters(parameters, ptypes, units)

        m = NFWPotential._vc_rs_rref_to_m(parameters['v_c'], parameters['r_s'], parameters['r_ref'])
        m = m.to(units['mass'])

        return NFWPotential(m=m, r_s=r_s, a=a, b=b, c=c, units=units, origin=origin)

    @staticmethod
    def _vc_rs_rref_to_m(v_c, r_s, r_ref):
        uu = r_ref / r_s
        vs2 = v_c**2 / uu / (np.log(1+uu)/uu**2 - 1/(uu*(1+uu)))
        return (r_s*vs2 / G)

# TODO: remove these in next full version
class SphericalNFWPotential(NFWPotential):

    def __init__(self, v_c, r_s, units=None, origin=None):
        import warnings
        warnings.warn("This class is now superseded by the single interface to all NFW "
                      "potentials, `NFWPotential`. Use that instead.", DeprecationWarning)
        super(SphericalNFWPotential, self).__init__(v_c=v_c, r_s=r_s, units=units, origin=origin)

    def save(self, *args, **kwargs):
        raise NotImplementedError("Use NFWPotential instead!")

class FlattenedNFWPotential(NFWPotential):

    def __init__(self, v_c, r_s, q_z, units=None, origin=None):
        import warnings
        warnings.warn("This class is now superseded by the single interface to all NFW "
                      "potentials, `NFWPotential`. Use that instead.", DeprecationWarning)

        super(FlattenedNFWPotential, self).__init__(v_c=v_c, r_s=r_s, c=q_z, units=units,
                                                    origin=origin)

    def save(self, *args, **kwargs):
        raise NotImplementedError("Use NFWPotential instead!")


cdef class LogarithmicWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(logarithmic_value)
        self.cpotential.gradient[0] = <gradientfunc>(logarithmic_gradient)

class LogarithmicPotential(CPotentialBase):
    r"""
    LogarithmicPotential(v_c, r_h, q1, q2, q3, phi=0, theta=0, psi=0, units=None, origin=None)

    Triaxial logarithmic potential.

    .. math::

        \Phi(x,y,z) &= \frac{1}{2}v_{c}^2\ln((x/q_1)^2 + (y/q_2)^2 + (z/q_3)^2 + r_h^2)\\

    Parameters
    ----------
    v_c : :class:`~astropy.units.Quantity`, numeric [velocity]
        Circular velocity.
    r_h : :class:`~astropy.units.Quantity`, numeric [length]
        Scale radius.
    q1 : numeric
        Flattening in X.
    q2 : numeric
        Flattening in Y.
    q3 : numeric
        Flattening in Z.
    phi : `~astropy.units.Quantity`, numeric
        First euler angle in the z-x-z convention.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, v_c, r_h, q1, q2, q3, phi=0., units=None, origin=None):
        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['v_c'] = v_c
        ptypes['v_c'] = 'speed'

        parameters['r_h'] = r_h
        ptypes['r_h'] = 'length'

        parameters['q1'] = q1
        parameters['q2'] = q2
        parameters['q3'] = q3

        parameters['phi'] = phi
        ptypes['phi'] = 'angle'

        super(LogarithmicPotential, self).__init__(parameters=parameters,
                                                   parameter_physical_types=ptypes,
                                                   units=units,
                                                   origin=origin)

        if not isinstance(self.units, DimensionlessUnitSystem):
            if self.units['angle'] != u.radian:
                raise ValueError("Angle unit must be radian.")


cdef class LeeSutoTriaxialNFWWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(leesuto_value)
        self.cpotential.density[0] = <densityfunc>(leesuto_density)
        self.cpotential.gradient[0] = <gradientfunc>(leesuto_gradient)

class LeeSutoTriaxialNFWPotential(CPotentialBase):
    r"""
    LeeSutoTriaxialNFWPotential(v_c, r_s, a, b, c, units=None, origin=None)

    Approximation of a Triaxial NFW Potential with the flattening in the density,
    not the potential.
    See `Lee & Suto (2003) <http://adsabs.harvard.edu/abs/2003ApJ...585..151L>`_
    for details.

    Parameters
    ----------
    v_c : `~astropy.units.Quantity`, numeric [velocity]
        Circular velocity at the scale radius.
    r_h : `~astropy.units.Quantity`, numeric [length]
        Scale radius.
    a : numeric
        Major axis.
    b : numeric
        Intermediate axis.
    c : numeric
        Minor axis.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, v_c, r_s, a, b, c, units=None, origin=None):

        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['v_c'] = v_c
        ptypes['v_c'] = 'speed'

        parameters['r_s'] = r_s
        ptypes['r_s'] = 'length'

        parameters['a'] = a
        parameters['b'] = b
        parameters['c'] = c

        super(LeeSutoTriaxialNFWPotential, self).__init__(parameters=parameters,
                                                          parameter_physical_types=ptypes,
                                                          units=units,
                                                          origin=origin)


cdef class LongMuraliBarWrapper(CPotentialWrapper):

    def __init__(self, G, parameters, q0):
        self.init([G] + list(parameters), np.ascontiguousarray(q0))
        self.cpotential.value[0] = <energyfunc>(longmuralibar_value)
        self.cpotential.gradient[0] = <gradientfunc>(longmuralibar_gradient)

class LongMuraliBarPotential(CPotentialBase):
    r"""
    LongMuraliBarPotential(m, a, b, c, alpha=0, units=None, origin=None)

    A simple, triaxial model for a galaxy bar. This is a softened "needle"
    density distribution with an analytic potential form.
    See `Long & Murali (1992) <http://adsabs.harvard.edu/abs/1992ApJ...397...44L>`_
    for details.

    Parameters
    ----------
    m : `~astropy.units.Quantity`, numeric [mass]
        Mass scale.
    a : `~astropy.units.Quantity`, numeric [length]
        Bar half-length.
    b : `~astropy.units.Quantity`, numeric [length]
        Like the Miyamoto-Nagai ``b`` parameter.
    c : `~astropy.units.Quantity`, numeric [length]
        Like the Miyamoto-Nagai ``c`` parameter.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, m, a, b, c, alpha=0., units=None, origin=None):

        parameters = OrderedDict()
        ptypes = OrderedDict()

        parameters['m'] = m
        ptypes['m'] = 'mass'

        for name in 'abc':
            parameters[name] = eval(name)
            ptypes[name] = 'length'

        parameters['alpha'] = alpha
        ptypes['alpha'] = 'angle'

        super(LongMuraliBarPotential, self).__init__(parameters=parameters,
                                                     parameter_physical_types=ptypes,
                                                     units=units,
                                                     origin=origin)




# coding: utf-8
"""
    Test conversions in propermotion.py
"""

from __future__ import absolute_import, division, print_function


# Standard library
import tempfile

# Third-party
import astropy.coordinates as coord
import astropy.units as u
import numpy as np

# This package
from ..propermotion import transform_proper_motion, pm_gal_to_icrs, pm_icrs_to_gal

_txt = """ # from here: http://www.astrostudio.org/xhipreadme.html
# HIPID ra dec pmra pmdec pml pmb
1 0.00091185 1.08901332    -4.58    -1.61    -4.85     0.29
2 0.00379738 -19.49883738   179.70     1.40   104.99  -145.85
3 0.00500794 38.85928608     4.28    -3.42     3.44    -4.26
10241 32.92989805 -31.39849677    40.91    18.19   -24.11    37.72
10242 32.93085553 3.45271681    18.94    -1.75    17.12     8.29
10243 32.93296084 60.71236365     0.85    -1.23     1.18    -0.91
19265 61.93883644 80.97646929    -7.44    -2.57    -3.31    -7.14
19266 61.93960267 4.31970083    -7.87    -3.75    -1.68    -8.55
19267 61.94490325 2.72570815    11.89   -33.83    34.41   -10.08
"""

class TestPMConvert(object):

    def setup(self):
        with tempfile.NamedTemporaryFile() as temp:
            temp.write(_txt.encode('utf-8'))
            temp.flush()
            temp.seek(0)
            self.data = np.genfromtxt(temp, names=True, skip_header=1)

    def test_gal_to_icrs(self):

        # test single entry's
        for row in self.data:
            c = coord.SkyCoord(ra=row['ra']*u.deg, dec=row['dec']*u.deg)
            muad = [row['pmra'],row['pmdec']]*u.mas/u.yr
            mulb = [row['pml'],row['pmb']]*u.mas/u.yr
            c_gal = c.transform_to(coord.Galactic)

            trans_muad = transform_proper_motion(c_gal, mulb, coord.ICRS)
            assert np.allclose(muad.value, trans_muad.value, atol=1E-2)

        # multiple entries
        c = coord.SkyCoord(ra=self.data['ra']*u.deg, dec=self.data['dec']*u.deg)
        muad = np.vstack((self.data['pmra'],self.data['pmdec']))*u.mas/u.yr
        mulb = np.vstack((self.data['pml'],self.data['pmb']))*u.mas/u.yr

        trans_muad = transform_proper_motion(c.transform_to(coord.Galactic), mulb, coord.ICRS)
        assert np.allclose(muad.value, np.round(trans_muad.value,2), atol=1E-2)

        # compare with old functions
        trans_muad2 = pm_gal_to_icrs(c, mulb)
        assert np.allclose(trans_muad.value, trans_muad2.value, atol=1E-2)

    def test_icrs_to_gal(self):

        # test single entrys
        for row in self.data:
            c = coord.SkyCoord(ra=row['ra']*u.deg, dec=row['dec']*u.deg)
            muad = [row['pmra'],row['pmdec']]*u.mas/u.yr
            mulb = [row['pml'],row['pmb']]*u.mas/u.yr

            trans_mulb = transform_proper_motion(c, muad, coord.Galactic)
            assert np.allclose(mulb.value, trans_mulb.value, atol=1E-2)

        # multiple entries
        c = coord.SkyCoord(ra=self.data['ra']*u.deg, dec=self.data['dec']*u.deg)
        muad = np.vstack((self.data['pmra'],self.data['pmdec']))*u.mas/u.yr
        mulb = np.vstack((self.data['pml'],self.data['pmb']))*u.mas/u.yr

        trans_mulb = transform_proper_motion(c, muad, coord.Galactic)
        assert np.allclose(mulb.value, np.round(trans_mulb.value,2), atol=1E-2)

        # compare with old functions
        trans_mulb2 = pm_icrs_to_gal(c, muad)
        assert np.allclose(trans_mulb.value, trans_mulb2.value, atol=1E-2)


def test_arbitrary_shape():
    shape = (128, 10)
    random_ra = np.random.uniform(0,2*np.pi,shape)
    random_dec = np.random.uniform(-np.pi/2.,np.pi/2.,shape)

    random_mua = np.random.normal(0,2,shape)
    random_mud = np.random.normal(0,2,shape)

    c = coord.SkyCoord(ra=random_ra*u.deg, dec=random_dec*u.deg)
    muad = np.vstack((random_mua[None],random_mud[None]))*u.mas/u.yr

    trans_mulb = transform_proper_motion(c, muad, coord.Galactic)
    assert trans_mulb[0].shape == shape



# coding: utf-8

from __future__ import division, print_function


# Third-party
import astropy.units as u
import numpy as np

# Project
# from .cpotential import CCompositePotential
# from ..core import CompositePotential
from .cybuiltin import (HernquistPotential,
                        MiyamotoNagaiPotential,
                        LogarithmicPotential,
                        NFWPotential)
from ..ccompositepotential import CCompositePotential
from ....units import galactic

__all__ = ['LM10Potential', 'MilkyWayPotential']

class LM10Potential(CCompositePotential):
    """
    The Galactic potential used by Law and Majewski (2010) to represent
    the Milky Way as a three-component sum of disk, bulge, and halo.

    The disk potential is an axisymmetric
    :class:`~gala.potential.MiyamotoNagaiPotential`, the bulge potential
    is a spherical :class:`~gala.potential.HernquistPotential`, and the
    halo potential is a triaxial :class:`~gala.potential.LogarithmicPotential`.

    Default parameters are fixed to those found in LM10 by fitting N-body
    simulations to the Sagittarius stream.

    Parameters
    ----------
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.
    disk : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.MiyamotoNagaiPotential`.
    bulge : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.HernquistPotential`.
    halo : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.LogarithmicPotential`.

    Note: in subclassing, order of arguments must match order of potential
    components added at bottom of init.
    """
    def __init__(self, units=galactic,
                 disk=dict(), bulge=dict(), halo=dict()):

        default_disk = dict(m=1E11*u.Msun, a=6.5*u.kpc, b=0.26*u.kpc)
        default_bulge = dict(m=3.4E10*u.Msun, c=0.7*u.kpc)
        default_halo = dict(q1=1.38, q2=1., q3=1.36, r_h=12.*u.kpc,
                            phi=97*u.degree,
                            v_c=np.sqrt(2)*121.858*u.km/u.s)

        for k,v in default_disk.items():
            if k not in disk:
                disk[k] = v

        for k,v in default_bulge.items():
            if k not in bulge:
                bulge[k] = v

        for k,v in default_halo.items():
            if k not in halo:
                halo[k] = v

        super(LM10Potential,self).__init__()

        self["disk"] = MiyamotoNagaiPotential(units=units, **disk)
        self["bulge"] = HernquistPotential(units=units, **bulge)
        self["halo"] = LogarithmicPotential(units=units, **halo)
        self.lock = True


class MilkyWayPotential(CCompositePotential):
    """
    A simple mass-model for the Milky Way consisting of a spherical nucleus and
    bulge, a Miyamoto-Nagai disk, and a spherical NFW dark matter halo.

    The disk model is taken from `Bovy (2015)
    <https://ui.adsabs.harvard.edu/#abs/2015ApJS..216...29B/abstract>`_ - if you
    use this potential, please also cite that work.

    Default parameters are fixed by fitting to a compilation of recent mass
    measurements of the Milky Way, from 10 pc to ~150 pc.

    Parameters
    ----------
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.
    disk : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.MiyamotoNagaiPotential`.
    bulge : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.HernquistPotential`.
    halo : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.LogarithmicPotential`.
    nucleus : dict (optional)
        Parameters to be passed to the :class:`~gala.potential.HernquistPotential`.

    Note: in subclassing, order of arguments must match order of potential
    components added at bottom of init.
    """
    def __init__(self, units=galactic,
                 disk=None, halo=None, bulge=None, nucleus=None):

        default_disk = dict(m=6.8E10*u.Msun, a=3.*u.kpc, b=0.28*u.kpc)
        default_bulge = dict(m=5E9*u.Msun, c=1.0*u.kpc)
        default_nucl = dict(m=1.71E9*u.Msun, c=0.07*u.kpc)
        default_halo = dict(m=5.4E11*u.Msun, r_s=15.62*u.kpc)

        if disk is None:
            disk = dict()

        if halo is None:
            halo = dict()

        if bulge is None:
            bulge = dict()

        if nucleus is None:
            nucleus = dict()

        for k,v in default_disk.items():
            if k not in disk:
                disk[k] = v

        for k,v in default_bulge.items():
            if k not in bulge:
                bulge[k] = v

        for k,v in default_halo.items():
            if k not in halo:
                halo[k] = v

        for k,v in default_nucl.items():
            if k not in nucleus:
                nucleus[k] = v

        super(MilkyWayPotential, self).__init__()

        self["disk"] = MiyamotoNagaiPotential(units=units, **disk)
        self["bulge"] = HernquistPotential(units=units, **bulge)
        self["nucleus"] = HernquistPotential(units=units, **nucleus)
        self["halo"] = NFWPotential(units=units, **halo)
        self.lock = True

# --------------------------------------------------------------------
# class TriaxialMWPotential(CCompositePotential):

#     def __init__(self, units=galactic,
#                  disk=dict(), bulge=dict(), halo=dict()):
#         """ Axis ratio values taken from Jing & Suto (2002). Other
#             parameters come from a by-eye fit to Bovy's MW2014Potential.
#             Choice of v_c sets circular velocity at Sun to 220 km/s
#         """

#         default_disk = dict(m=7E10, a=3.5, b=0.14)
#         default_bulge = dict(m=1E10, c=1.1)
#         default_halo = dict(a=1., b=0.75, c=0.55,
#                             v_c=0.239225, r_s=30.,
#                             phi=0., theta=0., psi=0.)

#         for k,v in default_disk.items():
#             if k not in disk:
#                 disk[k] = v

#         for k,v in default_bulge.items():
#             if k not in bulge:
#                 bulge[k] = v

#         for k,v in default_halo.items():
#             if k not in halo:
#                 halo[k] = v

#         kwargs = dict()
#         kwargs["disk"] = MiyamotoNagaiPotential(units=units, **disk)
#         kwargs["bulge"] = HernquistPotential(units=units, **bulge)
#         kwargs["halo"] = LeeSutoTriaxialNFWPotential(units=units, **halo)
#         super(TriaxialMWPotential,self).__init__(**kwargs)
# --------------------------------------------------------------------



from __future__ import absolute_import
from distutils.core import Extension
from astropy_helpers import setup_helpers

def get_extensions():
    exts = []

    # malloc
    mac_incl_path = "/usr/include/malloc"

    cfg = setup_helpers.DistutilsExtensionArgs()
    cfg['include_dirs'].append('numpy')
    cfg['include_dirs'].append(mac_incl_path)
    cfg['include_dirs'].append('gala/potential')
    cfg['extra_compile_args'].append('--std=gnu99')
    cfg['sources'].append('gala/potential/hamiltonian/chamiltonian.pyx')
    cfg['sources'].append('gala/potential/hamiltonian/src/chamiltonian.c')
    cfg['sources'].append('gala/potential/potential/src/cpotential.c')
    exts.append(Extension('gala.potential.hamiltonian.chamiltonian', **cfg))

    return exts

def get_package_data():

    return {'gala.potential.hamiltonian':
            ['src/*.h', 'src/*.c', '*.pyx', '*.pxd']}



# coding: utf-8

from __future__ import division, print_function


# Third-party
import numpy as np

__all__ = ['plot_projections']

def _get_axes(dim, subplots_kwargs=dict()):
    """
    Parameters
    ----------
    dim : int
        Dimensionality of the orbit.
    subplots_kwargs : dict (optional)
        Dictionary of kwargs passed to :func:`~matplotlib.pyplot.subplots`.
    """

    import matplotlib.pyplot as plt

    if dim > 1:
        n_panels = int(dim * (dim - 1) / 2)
    else:
        n_panels = 1
    figsize = subplots_kwargs.pop('figsize', (4*n_panels, 4))
    fig, axes = plt.subplots(1, n_panels, figsize=figsize,
                             **subplots_kwargs)

    if n_panels == 1:
        axes = [axes]

    else:
        axes = axes.flat

    return axes

def plot_projections(x, relative_to=None, autolim=True, axes=None,
                     subplots_kwargs=dict(), labels=None, plot_function=None,
                     **kwargs):
    """
    Given N-dimensional quantity, ``x``, make a figure containing 2D projections
    of all combinations of the axes.

    Parameters
    ----------
    x : array_like
        Array of values. ``axis=0`` is assumed to be the dimensionality,
        ``axis=1`` is the time axis. See :ref:`shape-conventions` for more
        information.
    relative_to : bool (optional)
        Plot the values relative to this value or values.
    autolim : bool (optional)
        Automatically set the plot limits to be something sensible.
    axes : array_like (optional)
        Array of matplotlib Axes objects.
    subplots_kwargs : dict (optional)
        Dictionary of kwargs passed to :func:`~matplotlib.pyplot.subplots`.
    labels : iterable (optional)
        List or iterable of axis labels as strings. They should correspond to
        the dimensions of the input orbit.
    plot_function : callable (optional)
        The ``matplotlib`` plot function to use. By default, this is
        :func:`~matplotlib.pyplot.scatter`, but can also be, e.g.,
        :func:`~matplotlib.pyplot.plot`.
    **kwargs
        All other keyword arguments are passed to the ``plot_function``.
        You can pass in any of the usual style kwargs like ``color=...``,
        ``marker=...``, etc.

    Returns
    -------
    fig : `~matplotlib.Figure`

    """

    # don't propagate changes back...
    x = np.array(x, copy=True)
    ndim = x.shape[0]

    # get axes object from arguments
    if axes is None:
        axes = _get_axes(dim=ndim, subplots_kwargs=subplots_kwargs)

    # if the quantities are relative
    if relative_to is not None:
        x -= relative_to

    # name of the plotting function
    plot_fn_name = plot_function.__name__

    # automatically determine limits
    if autolim:
        lims = []
        for i in range(ndim):
            max_,min_ = np.max(x[i]), np.min(x[i])
            delta = max_ - min_

            if delta == 0.:
                delta = 1.

            lims.append([min_ - delta*0.02, max_ + delta*0.02])

    k = 0
    for i in range(ndim):
        for j in range(ndim):
            if i >= j:
                continue # skip diagonal, upper triangle

            plot_func = getattr(axes[k], plot_fn_name)
            plot_func(x[i], x[j], **kwargs)

            if labels is not None:
                axes[k].set_xlabel(labels[i])
                axes[k].set_ylabel(labels[j])

            if autolim:
                axes[k].set_xlim(lims[i])
                axes[k].set_ylim(lims[j])

            k += 1

    axes[0].figure.tight_layout()
    return axes[0].figure



# coding: utf-8
"""
    Test the integrators.
"""

from __future__ import absolute_import, unicode_literals, division, print_function


# Third-party
import pytest
import matplotlib.pyplot as pl
import numpy as np

# Project
from .. import LeapfrogIntegrator, RK5Integrator, DOPRI853Integrator

# Integrators to test
integrator_list = [RK5Integrator, DOPRI853Integrator, LeapfrogIntegrator]

# ----------------------------------------------------------------------------

@pytest.mark.parametrize("Integrator", integrator_list)
def test_sho_forward_backward(Integrator):
    def sho(t,w,T):
        q,p = w
        return np.array([p, -(2*np.pi/T)**2*q])

    integrator = Integrator(sho, func_args=(1.,))

    dt = 0.01
    n_steps = 100
    if Integrator == LeapfrogIntegrator:
        dt = 1E-4
        n_steps = int(1E4)

    forw = integrator.run([0., 1.], dt=dt, n_steps=n_steps)
    back = integrator.run([0., 1.], dt=-dt, n_steps=n_steps)

    assert np.allclose(forw.w()[:,-1], back.w()[:,-1], atol=1E-6)

@pytest.mark.parametrize("Integrator", integrator_list)
def test_point_mass(Integrator):
    def F(t,w):
        x,y,px,py = w
        a = -1./(x*x+y*y)**1.5
        return np.array([px, py, x*a, y*a])

    q0 = np.array([1., 0.])
    p0 = np.array([0., 1.])
    T = 1.

    integrator = Integrator(F)
    orbit = integrator.run(np.append(q0,p0), t1=0., t2=2*np.pi, n_steps=1E4)

    assert np.allclose(orbit.w()[:,0], orbit.w()[:,-1], atol=1E-6)

@pytest.mark.parametrize("Integrator", integrator_list)
def test_point_mass_multiple(Integrator):
    def F(t,w):
        x,y,px,py = w
        a = -1/(x*x+y*y)**1.5
        return np.array([px, py, x*a, y*a])

    w0 = np.array([[1.0, 0.0, 0.0, 1.],
                   [0.8, 0.0, 0.0, 1.1],
                   [2., 1.0, -1.0, 1.1]]).T

    integrator = Integrator(F)
    orbit = integrator.run(w0, dt=1E-3, n_steps=1E4)

@pytest.mark.parametrize("Integrator", integrator_list)
def test_driven_pendulum(Integrator):
    def F(t,w,A,omega_d):
        q,p = w
        return np.array([p,-np.sin(q) + A*np.cos(omega_d*t)])

    integrator = Integrator(F, func_args=(0.07, 0.75))
    orbit = integrator.run([3., 0.], dt=1E-2, n_steps=1E4)

@pytest.mark.parametrize("Integrator", integrator_list)
def test_lorenz(Integrator):
    def F(t,w,sigma,rho,beta):
        x,y,z,px,py,pz = w
        return np.array([sigma*(y-x), x*(rho-z)-y, x*y-beta*z, 0., 0., 0.]).reshape(w.shape)

    sigma, rho, beta = 10., 28., 8/3.
    integrator = Integrator(F, func_args=(sigma, rho, beta))

    orbit = integrator.run([0.5,0.5,0.5,0,0,0], dt=1E-2, n_steps=1E4)

    # pl.plot(ws[0], ws[1])
    # pl.show()

@pytest.mark.parametrize("Integrator", integrator_list)
def test_memmap(tmpdir, Integrator):
    dt = 0.1
    n_steps = 1000
    nw0 = 10000
    mmap = np.memmap("/tmp/test_memmap.npy", mode='w+', shape=(2, n_steps+1, nw0))

    def sho(t,w,T):
        q,p = w
        return np.array([p, -(2*np.pi/T)**2*q])

    w0 = np.random.uniform(-1,1,size=(2,nw0))

    integrator = Integrator(sho, func_args=(1.,))

    orbit = integrator.run(w0, dt=dt, n_steps=n_steps, mmap=mmap)



# coding: utf-8

""" Astropy coordinate class for the Sagittarius coordinate system """

from __future__ import division, print_function


from astropy.coordinates import frame_transform_graph
import astropy.coordinates as coord
import astropy.units as u
from astropy.coordinates.matrix_utilities import rotation_matrix, matrix_product, matrix_transpose

__all__ = ["Orphan"]

class Orphan(coord.BaseCoordinateFrame):
    """
    A Heliocentric spherical coordinate system defined by the orbit
    of the Orphan stream, as described in
    Newberg et al. 2010 (see: `<http://arxiv.org/abs/1001.0576>`_).

    For more information about this class, see the Astropy documentation
    on coordinate frames in :mod:`~astropy.coordinates`.

    Parameters
    ----------
    representation : :class:`~astropy.coordinates.BaseRepresentation` or None
        A representation object or None to have no data (or use the other keywords)

    Lambda : angle_like, optional, must be keyword
        The longitude-like angle corresponding to Orphan's orbit.
    Beta : angle_like, optional, must be keyword
        The latitude-like angle corresponding to Orphan's orbit.
    distance : :class:`~astropy.units.Quantity`, optional, must be keyword
        The Distance for this object along the line-of-sight.

    pm_Lambda_cosBeta : :class:`~astropy.units.Quantity`, optional, must be keyword
        The proper motion in the longitude-like direction corresponding to
        the Orphan stream's orbit.
    pm_Beta : :class:`~astropy.units.Quantity`, optional, must be keyword
        The proper motion in the latitude-like direction perpendicular to the
        Orphan stream's orbit.
    radial_velocity : :class:`~astropy.units.Quantity`, optional, must be keyword
        The Distance for this object along the line-of-sight.

    """
    default_representation = coord.SphericalRepresentation
    default_differential = coord.SphericalCosLatDifferential

    frame_specific_representation_info = {
        coord.SphericalRepresentation: [
            coord.RepresentationMapping('lon', 'Lambda'),
            coord.RepresentationMapping('lat', 'Beta'),
            coord.RepresentationMapping('distance', 'distance')],
        coord.SphericalCosLatDifferential: [
            coord.RepresentationMapping('d_lon_coslat', 'pm_Lambda_cosBeta'),
            coord.RepresentationMapping('d_lat', 'pm_Beta'),
            coord.RepresentationMapping('d_distance', 'radial_velocity')],
        coord.SphericalDifferential: [
            coord.RepresentationMapping('d_lon', 'pm_Lambda'),
            coord.RepresentationMapping('d_lat', 'pm_Beta'),
            coord.RepresentationMapping('d_distance', 'radial_velocity')]
    }

    frame_specific_representation_info[coord.UnitSphericalRepresentation] = \
        frame_specific_representation_info[coord.SphericalRepresentation]
    frame_specific_representation_info[coord.UnitSphericalCosLatDifferential] = \
        frame_specific_representation_info[coord.SphericalCosLatDifferential]
    frame_specific_representation_info[coord.UnitSphericalDifferential] = \
        frame_specific_representation_info[coord.SphericalDifferential]

# Define the Euler angles
phi = 128.79 * u.degree
theta = 54.39 * u.degree
psi = 90.70 * u.degree

# Generate the rotation matrix using the x-convention (see Goldstein)
D = rotation_matrix(phi, "z")
C = rotation_matrix(theta, "x")
B = rotation_matrix(psi, "z")
R = matrix_product(B, C, D)

@frame_transform_graph.transform(coord.StaticMatrixTransform, coord.Galactic, Orphan)
def galactic_to_orp():
    """ Compute the transformation from Galactic spherical to
        heliocentric Orphan coordinates.
    """
    return R

# Oph to Galactic coordinates
@frame_transform_graph.transform(coord.StaticMatrixTransform, Orphan, coord.Galactic)
def oph_to_galactic():
    """ Compute the transformation from heliocentric Orphan coordinates to
        spherical Galactic.
    """
    return matrix_transpose(galactic_to_orp())



# coding: utf-8

""" Test helpers """

from __future__ import division, print_function


# Third-party
import astropy.coordinates as coord
import astropy.units as u
import matplotlib.pyplot as plt
import numpy as np

# Project
# from ..actionangle import classify_orbit
from ...units import galactic
from ...potential import HarmonicOscillatorPotential, IsochronePotential
from .._genfunc import genfunc_3d, solver, toy_potentials

def sanders_nvecs(N_max, dx, dy, dz):
    from itertools import product
    NNx = range(-N_max, N_max+1, dx)
    NNy = range(-N_max, N_max+1, dy)
    NNz = range(-N_max, N_max+1, dz)
    n_vectors = np.array([[i,j,k] for (i,j,k) in product(NNx,NNy,NNz)
                          if(not(i==0 and j==0 and k==0)            # exclude zero vector
                             and (k>0                               # northern hemisphere
                                  or (k==0 and j>0)                 # half of x-y plane
                                  or (k==0 and j==0 and i>0))       # half of x axis
                             and np.sqrt(i*i+j*j+k*k)<=N_max)])     # inside sphere
    return n_vectors

def sanders_act_ang_freq(t, w, circ, N_max=6):
    w2 = w.copy()

    if np.any(circ):
        w2[3:] = (w2[3:]*u.kpc/u.Myr).to(u.km/u.s).value
        (act,ang,n_vec,toy_aa,pars),loop2 = genfunc_3d.find_actions(w2.T, t/1000.,
                                                                    N_matrix=N_max, ifloop=True)
    else:
        (act,ang,n_vec,toy_aa,pars),loop2 = genfunc_3d.find_actions(w2.T, t,
                                                                    N_matrix=N_max, ifloop=True)

    actions = act[:3]
    angles = ang[:3]
    freqs = ang[3:6]

    if np.any(circ):
        toy_potential = IsochronePotential(m=pars[0]*1E11, b=pars[1], units=galactic)
        actions = (actions*u.kpc*u.km/u.s).to(u.kpc**2/u.Myr).value
        freqs = (freqs/u.Gyr).to(1/u.Myr).value
    else:
        toy_potential = HarmonicOscillatorPotential(omega=np.array(pars), units=galactic)

    return actions,angles,freqs,toy_potential

def _crazy_angle_loop(theta1,theta2,ax):
    cnt = 0
    ix1 = 0
    while True:
        cnt += 1

        for ix2 in range(ix1,ix1+1000):
            if ix2 > len(theta1)-1:
                ix2 = len(theta1)-1
                break

            if theta1[ix2] < theta1[ix1] or theta2[ix2] < theta2[ix1]:
                ix2 -= 1
                break

        if theta1[ix2] != theta1[ix1:ix2+1].max() or theta2[ix2] != theta2[ix1:ix2+1].max():
            ix1 = ix2+1
            continue

        if cnt > 100 or ix2 == len(theta1)-1:
            break

        if ix1 == ix2:
            ix1 = ix2+1
            continue

        ax.plot(theta1[ix1:ix2+1], theta2[ix1:ix2+1], alpha=0.5, marker='o', c='k')

        ix1 = ix2+1

def plot_angles(t, angles, freqs, subsample_factor=1000):
    theta = (angles[:,None] + freqs[:,None]*t[np.newaxis])
    subsample = theta.shape[1]//subsample_factor
#    subsample = 1
    theta = (theta[:,::subsample] / np.pi) % 2.
    print(theta.shape)
    fig,axes = plt.subplots(1,2,sharex=True,sharey=True,figsize=(10,5))
    # _crazy_angle_loop(theta[0], theta[1], axes[0])
    # _crazy_angle_loop(theta[0], theta[2], axes[1])
    axes[0].plot(theta[0], theta[1], ls='none')
    axes[0].plot(theta[0], theta[2], ls='none')

    axes[0].set_xlim(0,2)
    axes[0].set_ylim(0,2)
    return fig
    # axes[1].scatter(theta[0,ix], theta[2], alpha=0.5, marker='o', c=t)

def isotropic_w0(N=100):
    # positions
    d = np.random.lognormal(mean=np.log(25), sigma=0.5, size=N)
    phi = np.random.uniform(0, 2*np.pi, size=N)
    theta = np.arccos(np.random.uniform(size=N) - 0.5)

    vr = np.random.normal(150., 40., size=N)*u.km/u.s
    vt = np.random.normal(100., 40., size=N)
    vt = np.vstack((vt,np.zeros_like(vt))).T

    # rotate to be random position angle
    pa = np.random.uniform(0, 2*np.pi, size=N)
    M = np.array([[np.cos(pa), -np.sin(pa)],[np.sin(pa), np.cos(pa)]]).T
    vt = np.array([vv.dot(MM) for (vv,MM) in zip(vt,M)])*u.km/u.s
    vphi,vtheta = vt.T

    rep = coord.PhysicsSphericalRepresentation(r=d*u.dimensionless_unscaled,
                                               phi=phi*u.radian,
                                               theta=theta*u.radian)
    x = rep.represent_as(coord.CartesianRepresentation).xyz.T.value

    vr = vr.decompose(galactic).value * u.one
    vphi = vphi.decompose(galactic).value * u.one
    vtheta = vtheta.decompose(galactic).value * u.one

    vsph = coord.PhysicsSphericalDifferential(d_phi=vphi/(d*np.sin(theta)),
                                              d_theta=vtheta/d,
                                              d_r=vr)

    with u.set_enabled_equivalencies(u.dimensionless_angles()):
        v = vsph.represent_as(coord.CartesianDifferential,
                              base=rep).d_xyz.value.T

    return np.hstack((x,v)).T



# coding: utf-8

from __future__ import absolute_import, unicode_literals, division, print_function


# Third-party
import astropy.coordinates as coord
import astropy.units as u
from astropy.utils.data import get_pkg_data_filename
import numpy as np

# This package
from ..gd1 import GD1

def test_simple():
    c = coord.ICRS(coord.Angle(217.2141, u.degree),
                   coord.Angle(-11.4351, u.degree))
    c.transform_to(GD1)

    c = coord.Galactic(coord.Angle(217.2141, u.degree),
                       coord.Angle(-11.4351, u.degree))
    c.transform_to(GD1)

    c = GD1(217.2141*u.degree, -11.4351*u.degree)
    c.transform_to(coord.ICRS)
    c.transform_to(coord.Galactic)

    c = coord.Galactic(coord.Angle(217.2141, u.degree),
                       coord.Angle(-11.4351, u.degree))
    s = c.transform_to(GD1)

    # with distance
    c = GD1(coord.Angle(217.2141, u.degree),
            coord.Angle(-11.4351, u.degree),
            distance=15*u.kpc)
    c.transform_to(coord.ICRS)
    c2 = c.transform_to(coord.Galactic)
    assert np.allclose(c2.distance.value, c.distance.value)

def test_koposov():
    # Compare against Table 1 in Koposov et al. 2010

    filename = get_pkg_data_filename('gd1_coord.txt')
    k10_data = np.genfromtxt(filename, names=True, dtype=None)

    k10_icrs = coord.SkyCoord(ra=k10_data['ra'].astype(str),
                              dec=k10_data['dec'].astype(str),
                              unit=(u.hourangle, u.degree))

    k10_gd1 = GD1(phi1=k10_data['phi1']*u.degree,
                  phi2=k10_data['phi2']*u.degree)

    gala_gd1 = k10_icrs.transform_to(GD1)

    # TODO: why are these so different from the values in Koposov?
    assert np.allclose(k10_gd1.phi1.degree, gala_gd1.phi1.degree,
                       atol=1E-1)
    assert np.allclose(k10_gd1.phi2.degree, gala_gd1.phi2.degree,
                       atol=0.2)

    return
    # print(k10_gd1)
    # print("--")
    # print(gala_gd1)

    import matplotlib.pyplot as plt

    plt.figure(figsize=(5,5))
    plt.plot(k10_gd1.phi1.degree, gala_gd1.phi1.degree, ls='none')
    plt.xlim(min(k10_gd1.phi1.degree.min(),gala_gd1.phi1.degree.min()),
             max(k10_gd1.phi1.degree.max(),gala_gd1.phi1.degree.max()))
    plt.ylim(min(k10_gd1.phi1.degree.min(),gala_gd1.phi1.degree.min()),
             max(k10_gd1.phi1.degree.max(),gala_gd1.phi1.degree.max()))

    plt.figure(figsize=(5,5))
    plt.plot(k10_gd1.phi2.degree, gala_gd1.phi2.degree, ls='none')
    plt.xlim(min(k10_gd1.phi2.degree.min(),gala_gd1.phi2.degree.min()),
             max(k10_gd1.phi2.degree.max(),gala_gd1.phi2.degree.max()))
    plt.ylim(min(k10_gd1.phi2.degree.min(),gala_gd1.phi2.degree.min()),
             max(k10_gd1.phi2.degree.max(),gala_gd1.phi2.degree.max()))

    plt.show()




cdef extern from "potential/src/cpotential.h":
    ctypedef struct CPotential:
        pass

cdef void c_init_velocity(CPotential *p, int ndim, double t, double dt,
                          double *x_jm1, double *v_jm1, double *v_jm1_2, double *grad) nogil

cdef void c_leapfrog_step(CPotential *p, int ndim, double t, double dt,
                          double *x_jm1, double *v_jm1, double *v_jm1_2, double *grad) nogil



from __future__ import absolute_import

def get_package_data():
    return {'gala.coordinates': ['tests/idl_vgsr_vhel.txt',
                                 'tests/SgrCoord_data',
                                 'tests/gd1_coord.txt']}



# coding: utf-8

""" Astropy coordinate class for the Sagittarius coordinate system """

from __future__ import division, print_function


# Third-party
import numpy as np

from astropy.coordinates import frame_transform_graph
import astropy.coordinates as coord
import astropy.units as u
from astropy.coordinates.matrix_utilities import rotation_matrix, matrix_product, matrix_transpose

__all__ = ["Sagittarius"]

class Sagittarius(coord.BaseCoordinateFrame):
    """
    A Heliocentric spherical coordinate system defined by the orbit
    of the Sagittarius dwarf galaxy, as described in
    http://adsabs.harvard.edu/abs/2003ApJ...599.1082M
    and further explained in http://www.stsci.edu/~dlaw/Sgr/.

    Parameters
    ----------
    representation : `BaseRepresentation` or None
        A representation object or None to have no data (or use the other
        keywords).

    Lambda : `Angle`, optional, must be keyword
        The longitude-like angle corresponding to Sagittarius' orbit.
    Beta : `Angle`, optional, must be keyword
        The latitude-like angle corresponding to Sagittarius' orbit.
    distance : `Quantity`, optional, must be keyword
        The Distance for this object along the line-of-sight.

    pm_Lambda_cosBeta : :class:`~astropy.units.Quantity`, optional, must be keyword
        The proper motion along the stream in ``Lambda`` (including the
        ``cos(Beta)`` factor) for this object (``pm_Beta`` must also be given).
    pm_Beta : :class:`~astropy.units.Quantity`, optional, must be keyword
        The proper motion in Declination for this object (``pm_ra_cosdec`` must
        also be given).
    radial_velocity : :class:`~astropy.units.Quantity`, optional, must be keyword
        The radial velocity of this object.

    """
    default_representation = coord.SphericalRepresentation
    default_differential = coord.SphericalCosLatDifferential

    frame_specific_representation_info = {
        coord.SphericalRepresentation: [
            coord.RepresentationMapping('lon', 'Lambda'),
            coord.RepresentationMapping('lat', 'Beta'),
            coord.RepresentationMapping('distance', 'distance')],
        coord.SphericalCosLatDifferential: [
            coord.RepresentationMapping('d_lon_coslat', 'pm_Lambda_cosBeta'),
            coord.RepresentationMapping('d_lat', 'pm_Beta'),
            coord.RepresentationMapping('d_distance', 'radial_velocity')],
        coord.SphericalDifferential: [
            coord.RepresentationMapping('d_lon', 'pm_Lambda'),
            coord.RepresentationMapping('d_lat', 'pm_Beta'),
            coord.RepresentationMapping('d_distance', 'radial_velocity')]
    }

    frame_specific_representation_info[coord.UnitSphericalRepresentation] = \
        frame_specific_representation_info[coord.SphericalRepresentation]
    frame_specific_representation_info[coord.UnitSphericalCosLatDifferential] = \
        frame_specific_representation_info[coord.SphericalCosLatDifferential]
    frame_specific_representation_info[coord.UnitSphericalDifferential] = \
        frame_specific_representation_info[coord.SphericalDifferential]

# Define the Euler angles (from Law & Majewski 2010)
phi = (180+3.75) * u.degree
theta = (90-13.46) * u.degree
psi = (180+14.111534) * u.degree

# Generate the rotation matrix using the x-convention (see Goldstein)
D = rotation_matrix(phi, "z")
C = rotation_matrix(theta, "x")
B = rotation_matrix(psi, "z")
A = np.diag([1.,1.,-1.])
R = matrix_product(A, B, C, D)

# Galactic to Sgr coordinates
@frame_transform_graph.transform(coord.StaticMatrixTransform, coord.Galactic, Sagittarius)
def galactic_to_sgr():
    """ Compute the transformation from Galactic spherical to
        heliocentric Orphan coordinates.
    """
    return R

# Sgr to Galactic coordinates
@frame_transform_graph.transform(coord.StaticMatrixTransform, Sagittarius, coord.Galactic)
def sgr_to_galactic():
    """ Compute the transformation from heliocentric Orphan coordinates to
        spherical Galactic.
    """
    return matrix_transpose(galactic_to_sgr())



# coding: utf-8

from __future__ import division, print_function

import os
import warnings

# Third-party
import astropy.units as u
import numpy as np
import pytest

# Custom
from ....potential import Hamiltonian, NFWPotential, HernquistPotential
from ....dynamics import PhaseSpacePosition
from ....integrate import DOPRI853Integrator, LeapfrogIntegrator, RK5Integrator
from ....units import galactic

# Project
from ..core import mock_stream, streakline_stream, fardal_stream, dissolved_fardal_stream

@pytest.mark.parametrize("Integrator,kwargs",
                         zip([DOPRI853Integrator, LeapfrogIntegrator],
                             [dict(), dict()]))
def test_mock_stream(Integrator, kwargs):
    potential = NFWPotential.from_circular_velocity(v_c=0.2, r_s=20.,
                                                    units=galactic)
    ham = Hamiltonian(potential)

    w0 = PhaseSpacePosition(pos=[0.,15.,0]*u.kpc,
                            vel=[-0.13,0,0]*u.kpc/u.Myr)
    prog = ham.integrate_orbit(w0, dt=-2., n_steps=1023)
    prog = prog[::-1]

    k_mean = [1.,0.,0.,0.,1.,0.]
    k_disp = [0.,0.,0.,0.,0.,0.]
    stream = mock_stream(ham, prog, k_mean=k_mean, k_disp=k_disp,
                         prog_mass=1E4, Integrator=Integrator, **kwargs)

    # fig = prog.plot(subplots_kwargs=dict(sharex=False,sharey=False))
    # fig = stream.plot()

    # fig = prog.plot(subplots_kwargs=dict(sharex=False,sharey=False))
    # fig = stream.plot(color='#ff0000', alpha=0.5, axes=fig.axes)

    # pl.show()
    # return

    assert stream.pos.shape == (2048,) # two particles per step

    diff = np.abs(stream[-2:].xyz - prog[-1:].xyz)
    assert np.allclose(diff[0].value, 0.)
    assert np.allclose(diff[1,0].value, diff[1,1].value)
    assert np.allclose(diff[2].value, 0.)

mock_funcs = [streakline_stream, fardal_stream, dissolved_fardal_stream]
all_extra_args = [dict(), dict(), dict(t_disrupt=-250.*u.Myr)]
@pytest.mark.parametrize("mock_func, extra_kwargs", zip(mock_funcs, all_extra_args))
def test_each_type(mock_func, extra_kwargs):
    potential = NFWPotential.from_circular_velocity(v_c=0.2, r_s=20.,
                                                    units=galactic)
    ham = Hamiltonian(potential)

    w0 = PhaseSpacePosition(pos=[0.,15.,0]*u.kpc,
                            vel=[-0.13,0,0]*u.kpc/u.Myr)
    prog = ham.integrate_orbit(w0, dt=-2., n_steps=1023)
    prog = prog[::-1]

    stream = mock_func(ham, prog_orbit=prog, prog_mass=1E4,
                       Integrator=DOPRI853Integrator, **extra_kwargs)

    # import matplotlib.pyplot as plt
    # fig = prog.plot(subplots_kwargs=dict(sharex=False,sharey=False))
    # fig = stream.plot(color='#ff0000', alpha=0.5, axes=fig.axes)
    # fig = stream.plot()
    # plt.show()

    assert prog.t.shape == (1024,)
    assert stream.pos.shape == (2048,) # two particles per step

    # -----------------------
    # Test expected failures:

    # Deprecation warning for passing in potential
    warnings.simplefilter('always')
    with pytest.warns(DeprecationWarning):
        stream = mock_func(potential, prog_orbit=prog, prog_mass=1E4*u.Msun,
                           Integrator=DOPRI853Integrator, **extra_kwargs)

    # Integrator not supported
    with pytest.raises(ValueError):
        stream = mock_func(potential, prog_orbit=prog, prog_mass=1E4*u.Msun,
                           Integrator=RK5Integrator, **extra_kwargs)

    # Passed a phase-space position, not orbit
    with pytest.raises(TypeError):
        stream = mock_func(potential, prog_orbit=prog[0], prog_mass=1E4*u.Msun,
                           Integrator=DOPRI853Integrator, **extra_kwargs)

@pytest.mark.skipif('CI' in os.environ,
                    reason="For some reason, doesn't work on Travis/CI")
def test_animate(tmpdir):

    np.random.seed(42)
    pot = HernquistPotential(m=1E11, c=1., units=galactic)
    w0 = PhaseSpacePosition(pos=[5.,0,0]*u.kpc, vel=[0,0.1,0]*u.kpc/u.Myr)
    orbit = pot.integrate_orbit(w0, dt=1., n_steps=1000,
                                Integrator=DOPRI853Integrator)

    fardal_stream(pot, orbit, prog_mass=1E5*u.Msun, release_every=10,
                  snapshot_filename=os.path.join(str(tmpdir), "test.hdf5"),
                  seed=42)

    stream = fardal_stream(pot, orbit, prog_mass=1E5*u.Msun, release_every=10,
                           seed=42)

    import h5py
    with h5py.File(os.path.join(str(tmpdir), "test.hdf5")) as f:
        t = f['t'][:]
        pos = f['pos'][:]
        vel = f['vel'][:]

    assert np.allclose(t, orbit.t.value)

    for idx in range(pos.shape[2]):
        assert np.allclose(pos[:,-1,idx], stream.xyz.value[:,idx], rtol=1E-4)
        assert np.allclose(vel[:,-1,idx], stream.v_xyz.value[:,idx], rtol=1E-4)



# cdef extern from "potential/src/cpotential.h":
#     ctypedef struct CPotential:
#         pass

cdef extern from "src/funcdefs.h":
    ctypedef double (*densityfunc)(double t, double *pars, double *q) nogil
    ctypedef double (*energyfunc)(double t, double *pars, double *q) nogil
    ctypedef void (*gradientfunc)(double t, double *pars, double *q, double *grad) nogil
    ctypedef void (*hessianfunc)(double t, double *pars, double *q, double *hess) nogil

cdef extern from "potential/src/cpotential.h":
    enum:
        MAX_N_COMPONENTS = 16

    ctypedef struct CPotential:
        int n_components
        int n_dim
        densityfunc density[MAX_N_COMPONENTS]
        energyfunc value[MAX_N_COMPONENTS]
        gradientfunc gradient[MAX_N_COMPONENTS]
        hessianfunc hessian[MAX_N_COMPONENTS]
        int n_params[MAX_N_COMPONENTS]
        double *parameters[MAX_N_COMPONENTS]
        double *q0[MAX_N_COMPONENTS]

    double c_potential(CPotential *p, double t, double *q) nogil
    double c_density(CPotential *p, double t, double *q) nogil
    void c_gradient(CPotential *p, double t, double *q, double *grad) nogil
    void c_hessian(CPotential *p, double t, double *q, double *hess) nogil

    double c_d_dr(CPotential *p, double t, double *q, double *epsilon) nogil
    double c_d2_dr2(CPotential *p, double t, double *q, double *epsilon) nogil
    double c_mass_enclosed(CPotential *p, double t, double *q, double G, double *epsilon) nogil

cpdef _validate_pos_arr(double[:,::1] arr)

cdef class CPotentialWrapper:
    cdef CPotential cpotential
    cdef double[::1] _params
    cdef int[::1] _n_params
    cdef list _potentials # HACK: for CCompositePotentialWrapper
    cdef double[::1] _q0

    cpdef init(self, list parameters, double[::1] q0, int n_dim=?)

    cpdef energy(self, double[:,::1] q, double[::1] t)
    cpdef density(self, double[:,::1] q, double[::1] t)
    cpdef gradient(self, double[:,::1] q, double[::1] t)
    cpdef hessian(self, double[:,::1] q, double[::1] t)

    cpdef d_dr(self, double[:,::1] q, double G, double[::1] t)
    cpdef d2_dr2(self, double[:,::1] q, double G, double[::1] t)
    cpdef mass_enclosed(self, double[:,::1] q, double G, double[::1] t)



# coding: utf-8

from __future__ import division, print_function


# Standard library
import time

# Third-party
import matplotlib.pyplot as plt
import numpy as np
from scipy.misc import derivative
from astropy.extern.six.moves import cPickle as pickle
import pytest

# Project
from ..io import load
from ..core import CompositePotential
from ....units import UnitSystem, DimensionlessUnitSystem
from ....dynamics import PhaseSpacePosition
from ....integrate import LeapfrogIntegrator

def partial_derivative(func, point, dim_ix=0, **kwargs):
    xyz = np.array(point, copy=True)
    def wraps(a):
        xyz[dim_ix] = a
        return func(xyz)
    return derivative(wraps, point[dim_ix], **kwargs)

class PotentialTestBase(object):
    name = None
    potential = None # MUST SET THIS
    tol = 1E-5
    show_plots = False

    @classmethod
    def setup_class(cls):
        if cls.name is None:
            cls.name = cls.__name__[4:] # remove Test
        print("Testing potential: {}".format(cls.name))
        cls.w0 = np.array(cls.w0)
        cls.ndim = cls.w0.size // 2

        # TODO: need to test also quantity objects and phasespacepositions!

        # these are arrays we will test the methods on:
        w0_2d = np.repeat(cls.w0[:,None], axis=1, repeats=16)
        w0_3d = np.repeat(w0_2d[...,None], axis=2, repeats=8)
        w0_list = list(cls.w0)
        w0_slice = w0_2d[:,:4]
        cls.w0s = [cls.w0, w0_2d, w0_3d, w0_list, w0_slice]
        cls._grad_return_shapes = [cls.w0[:cls.ndim].shape + (1,),
                                   w0_2d[:cls.ndim].shape,
                                   w0_3d[:cls.ndim].shape,
                                   cls.w0[:cls.ndim].shape + (1,),
                                   w0_slice[:cls.ndim].shape]
        cls._hess_return_shapes = [(cls.ndim,) + cls.w0[:cls.ndim].shape + (1,),
                                   (cls.ndim,) + w0_2d[:cls.ndim].shape,
                                   (cls.ndim,) + w0_3d[:cls.ndim].shape,
                                   (cls.ndim,) + cls.w0[:cls.ndim].shape + (1,),
                                   (cls.ndim,) + w0_slice[:cls.ndim].shape]
        cls._valu_return_shapes = [x[1:] for x in cls._grad_return_shapes]

    def test_unitsystem(self):
        assert isinstance(self.potential.units, UnitSystem)

    def test_energy(self):
        assert self.ndim == self.potential.ndim

        for arr,shp in zip(self.w0s, self._valu_return_shapes):
            v = self.potential.energy(arr[:self.ndim])
            assert v.shape == shp

            g = self.potential.energy(arr[:self.ndim], t=0.1)
            g = self.potential.energy(arr[:self.ndim], t=0.1*self.potential.units['time'])

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            g = self.potential.energy(arr[:self.ndim], t=t)
            g = self.potential.energy(arr[:self.ndim], t=t*self.potential.units['time'])

    def test_gradient(self):
        for arr,shp in zip(self.w0s, self._grad_return_shapes):
            g = self.potential.gradient(arr[:self.ndim])
            assert g.shape == shp

            g = self.potential.gradient(arr[:self.ndim], t=0.1)
            g = self.potential.gradient(arr[:self.ndim], t=0.1*self.potential.units['time'])

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            g = self.potential.gradient(arr[:self.ndim], t=t)
            g = self.potential.gradient(arr[:self.ndim], t=t*self.potential.units['time'])

    def test_hessian(self):
        for arr,shp in zip(self.w0s, self._hess_return_shapes):
            g = self.potential.hessian(arr[:self.ndim])
            assert g.shape == shp

            g = self.potential.hessian(arr[:self.ndim], t=0.1)
            g = self.potential.hessian(arr[:self.ndim], t=0.1*self.potential.units['time'])

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            g = self.potential.hessian(arr[:self.ndim], t=t)
            g = self.potential.hessian(arr[:self.ndim], t=t*self.potential.units['time'])

    def test_mass_enclosed(self):
        for arr,shp in zip(self.w0s, self._valu_return_shapes):
            g = self.potential.mass_enclosed(arr[:self.ndim])
            assert g.shape == shp
            assert np.all(g > 0.)

            g = self.potential.mass_enclosed(arr[:self.ndim], t=0.1)
            g = self.potential.mass_enclosed(arr[:self.ndim], t=0.1*self.potential.units['time'])

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            g = self.potential.mass_enclosed(arr[:self.ndim], t=t)
            g = self.potential.mass_enclosed(arr[:self.ndim], t=t*self.potential.units['time'])

    def test_circular_velocity(self):
        for arr,shp in zip(self.w0s, self._valu_return_shapes):
            g = self.potential.circular_velocity(arr[:self.ndim])
            assert g.shape == shp
            assert np.all(g > 0.)

            g = self.potential.circular_velocity(arr[:self.ndim], t=0.1)
            g = self.potential.circular_velocity(arr[:self.ndim], t=0.1*self.potential.units['time'])

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            g = self.potential.circular_velocity(arr[:self.ndim], t=t)
            g = self.potential.circular_velocity(arr[:self.ndim], t=t*self.potential.units['time'])

    def test_repr(self):
        pot_repr = repr(self.potential)
        if isinstance(self.potential.units, DimensionlessUnitSystem):
            assert "dimensionless" in pot_repr
        else:
            assert str(self.potential.units['length']) in pot_repr
            assert str(self.potential.units['time']) in pot_repr
            assert str(self.potential.units['mass']) in pot_repr

        for k in self.potential.parameters.keys():
            assert "{}=".format(k) in pot_repr

    def test_compare(self):
        # skip if composite potentials
        if len(self.potential.parameters) == 0:
            return

        other = self.potential.__class__(units=self.potential.units, **self.potential.parameters)
        assert other == self.potential

        pars = self.potential.parameters.copy()
        for k in pars.keys():
            if k != 0:
                pars[k] = 1.1*pars[k]
        other = self.potential.__class__(units=self.potential.units, **pars)
        assert other != self.potential

    def test_plot(self):
        p = self.potential

        if self.show_plots:
            f = p.plot_contours(grid=(np.linspace(-10., 10., 100), 0., 0.),
                                labels=["X"])
            # f.suptitle("slice off from 0., won't have cusp")
            # f.savefig(os.path.join(plot_path, "contour_x.png"))

            f = p.plot_contours(grid=(np.linspace(-10., 10., 100),
                                      np.linspace(-10., 10., 100),
                                      0.),
                                cmap='Blues')
            # f.savefig(os.path.join(plot_path, "contour_xy.png"))

            f = p.plot_contours(grid=(np.linspace(-10., 10., 100),
                                      1.,
                                      np.linspace(-10., 10., 100)),
                                cmap='Blues', labels=["X", "Z"])
            # f.savefig(os.path.join(plot_path, "contour_xz.png"))

            plt.show()
            plt.close('all')

    def test_save_load(self, tmpdir):
        """
        Test writing to a YAML file, and reading back in
        """
        fn = str(tmpdir.join("{}.yml".format(self.name)))
        self.potential.save(fn)
        p = load(fn)
        p.energy(self.w0[:self.w0.size//2])
        p.gradient(self.w0[:self.w0.size//2])

    def test_numerical_gradient_vs_gradient(self):
        """
        Check that the value of the implemented gradient function is close to a
        numerically estimated value. This is to check the coded-up version.
        """

        dx = 1E-3 * np.sqrt(np.sum(self.w0[:self.w0.size//2]**2))
        max_x = np.sqrt(np.sum([x**2 for x in self.w0[:self.w0.size//2]]))

        grid = np.linspace(-max_x,max_x,8)
        grid = grid[grid != 0.]
        grids = [grid for i in range(self.w0.size//2)]
        xyz = np.ascontiguousarray(np.vstack(map(np.ravel, np.meshgrid(*grids))).T)

        def energy_wrap(xyz):
            xyz = np.ascontiguousarray(xyz[None])
            return self.potential._energy(xyz, t=np.array([0.]))[0]

        num_grad = np.zeros_like(xyz)
        for i in range(xyz.shape[0]):
            num_grad[i] = np.squeeze([partial_derivative(energy_wrap, xyz[i], dim_ix=dim_ix, n=1, dx=dx, order=5)
                                      for dim_ix in range(self.w0.size//2)])
        grad = self.potential._gradient(xyz, t=np.array([0.]))

        assert np.allclose(num_grad, grad, rtol=self.tol)

    def test_orbit_integration(self):
        """
        Make we can integrate an orbit in this potential
        """
        w0 = self.w0
        w0 = np.vstack((w0,w0,w0)).T

        t1 = time.time()
        orbit = self.potential.integrate_orbit(w0, dt=1., n_steps=10000,
                                               Integrator=LeapfrogIntegrator)
        print("Integration time (10000 steps): {}".format(time.time() - t1))

        if self.show_plots:
            f = orbit.plot()
            f.suptitle("Vector w0")
            plt.show()
            plt.close(f)

        us = self.potential.units
        w0 = PhaseSpacePosition(pos=w0[:self.ndim]*us['length'],
                                vel=w0[self.ndim:]*us['length']/us['time'])
        orbit = self.potential.integrate_orbit(w0, dt=1., n_steps=10000,
                                               Integrator=LeapfrogIntegrator)

        if self.show_plots:
            f = orbit.plot()
            f.suptitle("Object w0")
            plt.show()
            plt.close(f)

    def test_pickle(self, tmpdir):
        fn = str(tmpdir.join("{}.pickle".format(self.name)))
        with open(fn, "wb") as f:
            pickle.dump(self.potential, f)

        with open(fn, "rb") as f:
            p = pickle.load(f)

        p.energy(self.w0[:self.w0.size//2])

class CompositePotentialTestBase(PotentialTestBase):
    @pytest.mark.skip(reason="Skip composite potential repr test")
    def test_repr(self):
        pass

    @pytest.mark.skip(reason="Skip composite potential compare test")
    def test_compare(self):
        pass



# coding: utf-8
"""
    Test conversions in core.py
"""

from __future__ import absolute_import, division, print_function


# Standard library
import tempfile

# Third-party
import astropy.coordinates as coord
import astropy.units as u
from astropy.utils.data import get_pkg_data_filename
from astropy.tests.helper import quantity_allclose
import numpy as np

# This package
from ..velocity_frame_transforms import (vgal_to_hel, vhel_to_gal,
                                         vgsr_to_vhel, vhel_to_vgsr)

def test_vgsr_to_vhel():
    filename = get_pkg_data_filename('idl_vgsr_vhel.txt')
    data = np.genfromtxt(filename, names=True, skip_header=2)

    # one row
    row = data[0]
    l = coord.Angle(row["lon"] * u.degree)
    b = coord.Angle(row["lat"] * u.degree)
    c = coord.Galactic(l, b)
    vgsr = row["vgsr"] * u.km/u.s
    vlsr = [row["vx"],row["vy"],row["vz"]]*u.km/u.s # this is right
    vcirc = row["vcirc"]*u.km/u.s

    vsun = vlsr + [0,1,0]*vcirc
    vhel = vgsr_to_vhel(c, vgsr, vsun=vsun)
    return
    np.testing.assert_almost_equal(vhel.value, row['vhelio'], decimal=4)

    # now check still get right answer passing in ICRS coordinates
    vhel = vgsr_to_vhel(c.transform_to(coord.ICRS), vgsr, vsun=vsun)
    np.testing.assert_almost_equal(vhel.value, row['vhelio'], decimal=4)

    # all together now
    l = coord.Angle(data["lon"] * u.degree)
    b = coord.Angle(data["lat"] * u.degree)
    c = coord.Galactic(l, b)
    vgsr = data["vgsr"] * u.km/u.s
    vhel = vgsr_to_vhel(c, vgsr, vsun=vsun)
    np.testing.assert_almost_equal(vhel.value, data['vhelio'], decimal=4)

    # now check still get right answer passing in ICRS coordinates
    vhel = vgsr_to_vhel(c.transform_to(coord.ICRS), vgsr, vsun=vsun)
    np.testing.assert_almost_equal(vhel.value, data['vhelio'], decimal=4)

def test_vgsr_to_vhel_misc():
    # make sure it works with longitude in 0-360 or -180-180
    l1 = coord.Angle(190.*u.deg)
    l2 = coord.Angle(-170.*u.deg)
    b = coord.Angle(30.*u.deg)

    c1 = coord.Galactic(l1, b)
    c2 = coord.Galactic(l2, b)

    vgsr = -110.*u.km/u.s
    vhel1 = vgsr_to_vhel(c1, vgsr)
    vhel2 = vgsr_to_vhel(c2, vgsr)

    np.testing.assert_almost_equal(vhel1.value, vhel2.value, decimal=9)

def test_vhel_to_vgsr():
    filename = get_pkg_data_filename('idl_vgsr_vhel.txt')
    data = np.genfromtxt(filename, names=True, skip_header=2)

    # one row
    row = data[0]
    l = coord.Angle(row["lon"] * u.degree)
    b = coord.Angle(row["lat"] * u.degree)
    c = coord.Galactic(l, b)
    vhel = row["vhelio"] * u.km/u.s
    vlsr = [row["vx"],row["vy"],row["vz"]]*u.km/u.s # this is right
    vcirc = row["vcirc"]*u.km/u.s

    vsun = vlsr + [0,1,0]*vcirc
    vgsr = vhel_to_vgsr(c, vhel, vsun=vsun)
    np.testing.assert_almost_equal(vgsr.value, row['vgsr'], decimal=4)

    # now check still get right answer passing in ICRS coordinates
    vgsr = vhel_to_vgsr(c.transform_to(coord.ICRS), vhel, vsun=vsun)
    np.testing.assert_almost_equal(vgsr.value, row['vgsr'], decimal=4)

    # all together now
    l = coord.Angle(data["lon"] * u.degree)
    b = coord.Angle(data["lat"] * u.degree)
    c = coord.Galactic(l, b)
    vhel = data["vhelio"] * u.km/u.s
    vgsr = vhel_to_vgsr(c, vhel, vsun=vsun)
    np.testing.assert_almost_equal(vgsr.value, data['vgsr'], decimal=4)

    # now check still get right answer passing in ICRS coordinates
    vgsr = vhel_to_vgsr(c.transform_to(coord.ICRS), vhel, vsun=vsun)
    np.testing.assert_almost_equal(vgsr.value, data['vgsr'], decimal=4)

_txt = """# from: XHIP catalog
# ra dec HIPID l b dist pml pmb rv U V W
0.022010 20.036114      7 106.82021040 -41.22316218   57.56  -253.69  -138.84    8.30   71.7    2.1  -34.0
2.208349 40.494550    714 114.23363142 -21.65650026  249.00     5.57    -9.00  -11.78    0.1  -16.3   -5.5
3.126297 14.563522    999 108.98177530 -47.25067692   40.94   296.66  -141.05  -15.30  -44.5  -47.6   -7.3
"""
class TestVHelGalConvert(object):

    def setup(self):
        with tempfile.NamedTemporaryFile(mode='w+b') as temp:
            temp.write(_txt.encode('utf-8'))
            temp.flush()
            temp.seek(0)
            self.data = np.genfromtxt(temp, names=True, skip_header=1)

        # This should make the transformations more compatible
        g = coord.Galactic(l=0*u.deg, b=0*u.deg).transform_to(coord.ICRS)
        self.galcen_frame = coord.Galactocentric(galcen_coord=g,
                                                 z_sun=0*u.kpc)

    def test_vhel_to_gal_single(self):

        for row in self.data: # test one entry at a time
            c = coord.SkyCoord(ra=row['ra']*u.deg, dec=row['dec']*u.deg,
                               distance=row['dist']*u.pc)
            icrs = c.icrs
            gal = c.galactic
            pm = [row['pml']*u.mas/u.yr, row['pmb']*u.mas/u.yr,]
            rv = row['rv']*u.km/u.s

            # stupid check
            vxyz_i = vhel_to_gal(icrs, pm=pm, rv=rv,
                                 vcirc=0*u.km/u.s,
                                 vlsr=[0.,0,0]*u.km/u.s)

            vxyz = vhel_to_gal(gal, pm=pm, rv=rv,
                               vcirc=0*u.km/u.s,
                               vlsr=[0.,0,0]*u.km/u.s)

            assert vxyz_i.shape == vxyz.shape

            true_UVW = np.array([row['U'], row['V'], row['W']])
            UVW = vxyz.to(u.km/u.s).value

            # catalog values are rounded
            assert np.allclose(UVW, true_UVW, rtol=1E-2, atol=0.1)

        # --------------------------------------------------------------------
        # l = 0
        # without LSR and circular velocity
        c = coord.SkyCoord(ra=self.galcen_frame.galcen_ra,
                           dec=self.galcen_frame.galcen_dec,
                           distance=2*u.kpc)
        pm = [0,0]*u.mas/u.yr
        rv = 20*u.km/u.s
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=rv,
                           vcirc=0*u.km/u.s,
                           vlsr=[0.,0,0]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)
        assert np.allclose(vxyz.to(u.km/u.s).value, [20,0,0.],
                           atol=1E-12)

        # with LSR and circular velocity
        c = coord.SkyCoord(ra=self.galcen_frame.galcen_ra,
                           dec=self.galcen_frame.galcen_dec,
                           distance=2*u.kpc)
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=rv,
                           vcirc=200*u.km/u.s,
                           vlsr=[-20.,0,10]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)
        assert np.allclose(vxyz.to(u.km/u.s).value, [0,200,10],
                           atol=1E-12)

        # l = 90
        # with LSR and circular velocity
        c = coord.SkyCoord(l=90*u.deg, b=0*u.deg,
                           distance=2*u.kpc, frame=coord.Galactic)
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=rv,
                           vcirc=200*u.km/u.s,
                           vlsr=[-20.,0,10]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)
        assert np.allclose(vxyz.to(u.km/u.s).value, [-20,220,10],
                           atol=1E-5)

        # l = 180
        # with LSR and circular velocity
        c = coord.SkyCoord(l=180*u.deg, b=0*u.deg,
                           distance=2*u.kpc, frame=coord.Galactic)
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=rv,
                           vcirc=200*u.km/u.s,
                           vlsr=[-20.,0,10]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)
        assert np.allclose(vxyz.to(u.km/u.s).value, [-40,200,10],
                           atol=1E-12)

        # l = 270
        # with LSR and circular velocity
        c = coord.SkyCoord(l=270*u.deg, b=0*u.deg,
                           distance=2*u.kpc, frame=coord.Galactic)
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=rv,
                           vcirc=200*u.km/u.s,
                           vlsr=[-20.,0,10]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)
        assert np.allclose(vxyz.to(u.km/u.s).value, [-20,180,10],
                           atol=1E-5)

    def test_vhel_to_gal_array(self):
        # test all together
        d = self.data
        c = coord.SkyCoord(ra=d['ra']*u.deg, dec=d['dec']*u.deg,
                           distance=d['dist']*u.pc)
        icrs = c.icrs
        gal = c.galactic
        pm = [d['pml'], d['pmb']]*u.mas/u.yr
        rv = d['rv']*u.km/u.s

        # stupid check
        vxyz_i = vhel_to_gal(icrs, pm=pm, rv=rv,
                             vcirc=0*u.km/u.s,
                             vlsr=[0.,0,0]*u.km/u.s)
        vxyz = vhel_to_gal(gal, pm=pm, rv=rv,
                           vcirc=0*u.km/u.s,
                           vlsr=[0.,0,0]*u.km/u.s)
        assert vxyz_i.shape == vxyz.shape

        # check values
        true_UVW = np.array([d['U'], d['V'], d['W']])
        UVW = vxyz.to(u.km/u.s).value

        # catalog values are rounded
        assert np.allclose(UVW, true_UVW, rtol=1E-2, atol=0.1)

    def test_vgal_to_hel_single(self):

        for row in self.data: # test one entry at a time
            c = coord.SkyCoord(ra=row['ra']*u.deg, dec=row['dec']*u.deg,
                               distance=row['dist']*u.pc)
            gal = c.galactic
            vxyz = [row['U'], row['V'], row['W']] * u.km/u.s

            vhel = vgal_to_hel(gal, vxyz,
                               vcirc=0.*u.km/u.s,
                               vlsr=[0.,0,0]*u.km/u.s,
                               galactocentric_frame=self.galcen_frame)

            # tolerance set by the catalog rounded numbers
            assert quantity_allclose(vhel[0], row['pml'] * u.mas/u.yr, rtol=1E-2)
            assert quantity_allclose(vhel[1], row['pmb'] * u.mas/u.yr, rtol=1E-2)
            assert quantity_allclose(vhel[2], row['rv'] * u.km/u.s, rtol=1E-2)

        # --------------------------------------------------------------------
        # l = 0
        # without LSR and circular velocity
        c = coord.SkyCoord(l=0*u.deg, b=0*u.deg, distance=2*u.kpc,
                           frame=coord.Galactic)
        vxyz = [20., 0, 0]*u.km/u.s
        vhel = vgal_to_hel(c.galactic, vxyz,
                           vcirc=0*u.km/u.s,
                           vlsr=[0.,0,0]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)

        assert np.allclose(vhel[0].value, 0., atol=1E-12)
        assert np.allclose(vhel[1].value, 0., atol=1E-12)
        assert np.allclose(vhel[2].to(u.km/u.s).value, 20., atol=1E-12)

        vxyz = [20., 0, 50]*u.km/u.s
        vhel = vgal_to_hel(c.galactic, vxyz,
                           vcirc=0*u.km/u.s,
                           vlsr=[0.,0,0]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)

        assert np.allclose(vhel[0].value, 0., atol=2E-5) # TODO: astropy precision issues
        with u.set_enabled_equivalencies(u.dimensionless_angles()):
            assert quantity_allclose(vhel[1], 50*u.km/u.s / (2*u.kpc),
                                     atol=1E-10*u.mas/u.yr)
        assert quantity_allclose(vhel[2].to(u.km/u.s), vxyz[0],
                                 atol=1E-10*u.km/u.s)

        # with LSR and circular velocity
        vxyz = [20., 0, 50]*u.km/u.s
        vhel = vgal_to_hel(c.galactic, vxyz,
                           vcirc=-200*u.km/u.s,
                           vlsr=[0., 0, 10]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)

        with u.set_enabled_equivalencies(u.dimensionless_angles()):
            assert quantity_allclose(vhel[0],
                                     (200.*u.km/u.s) / (2*u.kpc),
                                     atol=1E-10*u.mas/u.yr)
            assert quantity_allclose(vhel[1],
                                     (40.*u.km/u.s) / (2*u.kpc),
                                     atol=1E-6*u.mas/u.yr)

        assert quantity_allclose(vhel[2], 20.*u.km/u.s,
                                 atol=1E-10*u.km/u.s)

    def test_vgal_to_hel_array(self):
        # test all together
        d = self.data
        c = coord.SkyCoord(ra=d['ra']*u.deg, dec=d['dec']*u.deg,
                           distance=d['dist']*u.pc)

        pm = np.vstack([d['pml'],d['pmb']])*u.mas/u.yr
        rv = d['rv']*u.km/u.s

        vxyz = np.vstack((d['U'], d['V'], d['W']))*u.km/u.s
        vhel = vgal_to_hel(c.galactic, vxyz,
                           vcirc=0.*u.km/u.s,
                           vlsr=[0.,0,0]*u.km/u.s,
                           galactocentric_frame=self.galcen_frame)

        # tolerance set by the catalog rounded numbers
        assert quantity_allclose(vhel[0], pm[0], rtol=1E-2)
        assert quantity_allclose(vhel[1], pm[1], rtol=1E-2)
        assert quantity_allclose(vhel[2], rv, rtol=5E-3)

    def test_roundtrip_icrs(self):
        np.random.seed(42)
        n = 100

        # yeahhhh, i know this isn't uniform on the sphere - shut up
        c = coord.SkyCoord(ra=np.random.uniform(0,360,n)*u.degree,
                           dec=np.random.uniform(-90,90,n)*u.degree,
                           distance=np.random.uniform(0.1,10.,n)*u.kpc)

        pm = np.random.uniform(-20,20,size=(2,n)) * u.mas/u.yr
        vr = np.random.normal(0., 75., size=n)*u.km/u.s

        # first to galactocentric
        vxyz = vhel_to_gal(c.icrs, pm=pm, rv=vr)

        # then back again, wooo
        vhel2 = vgal_to_hel(c.icrs, vxyz)

        assert quantity_allclose(vhel2[0], pm[0], rtol=1e-12)
        assert quantity_allclose(vhel2[1], pm[1], rtol=1e-12)
        assert quantity_allclose(vhel2[2], vr, rtol=1e-12)

    def test_roundtrip_gal(self):
        np.random.seed(42)
        n = 100

        # yeahhhh, i know this isn't uniform on the sphere - shut up
        c = coord.SkyCoord(ra=np.random.uniform(0,360,n)*u.degree,
                           dec=np.random.uniform(-90,90,n)*u.degree,
                           distance=np.random.uniform(0.1,10.,n)*u.kpc)

        pm = np.random.uniform(-20,20,size=(2,n)) * u.mas/u.yr
        vr = np.random.normal(0., 75., size=n)*u.km/u.s

        # first to galactocentric
        vxyz = vhel_to_gal(c.galactic, pm=pm, rv=vr)

        # then back again, wooo
        vhel2 = vgal_to_hel(c.galactic, vxyz)

        # TODO: why such bad roundtripping???
        assert quantity_allclose(vhel2[0], pm[0], rtol=1e-12)
        assert quantity_allclose(vhel2[1], pm[1], rtol=1e-12)
        assert quantity_allclose(vhel2[2], vr, rtol=1e-12)



# coding: utf-8

""" Test ...  """

from __future__ import division, print_function


# Third-party
import numpy as np
from astropy import log as logger
import astropy.units as u

# Project
from ..analyticactionangle import isochrone_to_aa, harmonic_oscillator_to_aa
from ...potential import IsochronePotential, HarmonicOscillatorPotential
from ...units import galactic
from ...util import assert_angles_allclose
from .helpers import toy_potentials

class TestIsochrone(object):

    def setup(self):
        logger.info("======== Isochrone ========")
        self.N = 100
        np.random.seed(42)
        x = np.random.uniform(-10., 10., size=(3,self.N))
        v = np.random.uniform(-1., 1., size=(3,self.N)) / 33.
        w0 = np.vstack((x,v))

        self.potential = IsochronePotential(units=galactic, m=1.E11, b=5.)
        self.w = self.potential.integrate_orbit(w0, dt=0.1, n_steps=10000)
        self.w = self.w[::10]

    def test(self):
        for n in range(self.N):
            logger.debug("Orbit {}".format(n))

            actions,angles,freqs = isochrone_to_aa(self.w[:,n], self.potential)
            actions = actions.value
            angles = angles.value

            for i in range(3):
                assert np.allclose(actions[i,1:], actions[i,0], rtol=1E-5)

            # Compare to genfunc
            x = self.w.xyz.value[...,n]
            v = self.w.v_xyz.value[...,n]
            s_v = (v*u.kpc/u.Myr).to(u.km/u.s).value
            s_w = np.vstack((x,s_v))
            m = self.potential.parameters['m'].value / 1E11
            b = self.potential.parameters['b'].value
            aa = np.array([toy_potentials.angact_iso(s_w[:,i].T, params=(m,b)) for i in range(s_w.shape[1])])
            s_actions = (aa[:,:3]*u.km/u.s*u.kpc).decompose(galactic).value
            s_angles = aa[:,3:]

            assert np.allclose(actions, s_actions.T, rtol=1E-8)
            assert_angles_allclose(angles, s_angles.T, rtol=1E-8)

            # TODO: when I fix actionangle -> xv function, re-add this
            # x2,v2 = isochrone_aa_to_xv(actions, angles, self.potential)

            # assert np.allclose(x, x2, rtol=1E-8)
            # assert np.allclose(v, v2, rtol=1E-8)

class TestHarmonicOscillator(object):

    def setup(self):
        logger.info("======== Harmonic Oscillator ========")
        self.N = 100
        np.random.seed(42)
        x = np.random.uniform(-10., 10., size=(3,self.N))
        v = np.random.uniform(-1., 1., size=(3,self.N)) / 33.
        w0 = np.vstack((x,v))

        self.potential = HarmonicOscillatorPotential(omega=np.array([0.013, 0.02, 0.005]), units=galactic)
        self.w = self.potential.integrate_orbit(w0, dt=0.1, n_steps=10000)
        self.w = self.w[::10]

    def test(self):
        """
            !!!!! NOTE !!!!!
            For Harmonic Oscillator, Sanders' code works for the units I use...
        """
        for n in range(self.N):
            logger.debug("Orbit {}".format(n))

            actions,angles,freq = harmonic_oscillator_to_aa(self.w[:,n], self.potential)
            actions = actions.value
            angles = angles.value

            for i in range(3):
                assert np.allclose(actions[i,1:], actions[i,0], rtol=1E-5)

            # Compare to genfunc
            x = self.w.xyz.value[...,n]
            v = self.w.v_xyz.value[...,n]
            s_w = np.vstack((x,v))
            omega = self.potential.parameters['omega'].value
            aa = np.array([toy_potentials.angact_ho(s_w[:,i].T, omega=omega) for i in range(s_w.shape[1])])
            s_actions = aa[:,:3]
            s_angles = aa[:,3:]

            assert np.allclose(actions, s_actions.T, rtol=1E-8)
            assert_angles_allclose(angles, s_angles.T, rtol=1E-8)

            # test roundtrip
            # x2,v2 = harmonic_oscillator_aa_to_xv(actions, angles, self.potential)
            # TODO: transform back??



# coding: utf-8

from __future__ import division, print_function

# Third-party
import astropy.units as u
import numpy as np

# Project
from ....dynamics import PhaseSpacePosition, Orbit
from ....units import galactic
PSP = PhaseSpacePosition
ORB = Orbit

class _TestBase(object):
    use_half_ndim = False
    E_unit = u.erg/u.kg

    @classmethod
    def setup_class(cls):
        np.random.seed(42)

        ndim = 6
        r_ndim = ndim # return ndim
        if cls.use_half_ndim:
            r_ndim = r_ndim // 2
        norbits = 16
        ntimes = 8

        # some position or phase-space position arrays we will test methods on:
        cls.w0s = []
        cls.energy_return_shapes = []
        cls.gradient_return_shapes = []
        cls.hessian_return_shapes = []

        # 1D - phase-space position
        cls.w0s.append(PSP(pos=np.random.random(size=ndim//2),
                           vel=np.random.random(size=ndim//2)))
        cls.w0s.append(PSP(pos=np.random.random(size=ndim//2)*u.kpc,
                           vel=np.random.random(size=ndim//2)*u.km/u.s))
        cls.energy_return_shapes += [(1,)]*2
        cls.gradient_return_shapes += [(r_ndim,1)]*2
        cls.hessian_return_shapes += [(r_ndim,r_ndim,1)]*2

        # 2D - phase-space position
        cls.w0s.append(PSP(pos=np.random.random(size=(ndim//2,norbits)),
                           vel=np.random.random(size=(ndim//2,norbits))))
        cls.w0s.append(PSP(pos=np.random.random(size=(ndim//2,norbits))*u.kpc,
                           vel=np.random.random(size=(ndim//2,norbits))*u.km/u.s))
        cls.energy_return_shapes += [(norbits,)]*2
        cls.gradient_return_shapes += [(r_ndim,norbits)]*2
        cls.hessian_return_shapes += [(r_ndim,r_ndim,norbits)]*2

        # 3D - phase-space position
        cls.w0s.append(PSP(pos=np.random.random(size=(ndim//2,norbits,ntimes)),
                           vel=np.random.random(size=(ndim//2,norbits,ntimes))))
        cls.w0s.append(PSP(pos=np.random.random(size=(ndim//2,norbits,ntimes))*u.kpc,
                           vel=np.random.random(size=(ndim//2,norbits,ntimes))*u.km/u.s))
        cls.energy_return_shapes += [(norbits,ntimes)]*2
        cls.gradient_return_shapes += [(r_ndim,norbits,ntimes)]*2
        cls.hessian_return_shapes += [(r_ndim,r_ndim,norbits,ntimes)]*2

        # 2D - orbit
        cls.w0s.append(ORB(pos=np.random.random(size=(ndim//2,ntimes)),
                           vel=np.random.random(size=(ndim//2,ntimes))))
        cls.w0s.append(ORB(pos=np.random.random(size=(ndim//2,ntimes))*u.kpc,
                           vel=np.random.random(size=(ndim//2,ntimes))*u.km/u.s))
        cls.energy_return_shapes += [(ntimes,)]*2
        cls.gradient_return_shapes += [(r_ndim,ntimes,)]*2
        cls.hessian_return_shapes += [(r_ndim,r_ndim,ntimes,)]*2

        # 3D - orbit
        cls.w0s.append(ORB(pos=np.random.random(size=(ndim//2,ntimes,norbits)),
                           vel=np.random.random(size=(ndim//2,ntimes,norbits))))
        cls.w0s.append(ORB(pos=np.random.random(size=(ndim//2,ntimes,norbits))*u.kpc,
                           vel=np.random.random(size=(ndim//2,ntimes,norbits))*u.km/u.s))
        cls.energy_return_shapes += [(ntimes,norbits)]*2
        cls.gradient_return_shapes += [(r_ndim,ntimes,norbits)]*2
        cls.hessian_return_shapes += [(r_ndim,r_ndim,ntimes,norbits)]*2

        _obj_w0s = cls.w0s[:]
        for w0,eshp,gshp,hshp in zip(_obj_w0s,
                                     cls.energy_return_shapes,
                                     cls.gradient_return_shapes,
                                     cls.hessian_return_shapes):
            cls.w0s.append(w0.w(galactic))
            cls.energy_return_shapes.append(eshp)
            cls.gradient_return_shapes.append(gshp)
            cls.hessian_return_shapes.append(hshp)

    def test_energy(self):
        for arr,shp in zip(self.w0s, self.energy_return_shapes):
            if self.E_unit.is_equivalent(u.one) and hasattr(arr, 'pos') and \
                not arr.xyz.unit.is_equivalent(u.one):
                continue

            v = self.obj.energy(arr)
            assert v.shape == shp
            assert v.unit.is_equivalent(self.E_unit)

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            self.obj.energy(arr, t=0.1)
            self.obj.energy(arr, t=t)
            self.obj.energy(arr, t=0.1*self.obj.units['time'])

    def test_gradient(self):
        for arr,shp in zip(self.w0s, self.gradient_return_shapes):
            if self.E_unit.is_equivalent(u.one) and hasattr(arr, 'pos') and \
                not arr.xyz.unit.is_equivalent(u.one):
                continue

            v = self.obj.gradient(arr)
            assert v.shape == shp
            # TODO: check return units

            t = np.zeros(np.array(arr).shape[1:]) + 0.1
            self.obj.gradient(arr, t=0.1)
            self.obj.gradient(arr, t=t)
            self.obj.gradient(arr, t=0.1*self.obj.units['time'])

    def test_hessian(self):
        for arr,shp in zip(self.w0s, self.hessian_return_shapes):
            if self.E_unit.is_equivalent(u.one) and hasattr(arr, 'pos') and \
                not arr.xyz.unit.is_equivalent(u.one):
                continue

            g = self.obj.hessian(arr)
            assert g.shape == shp
            # TODO: check return units



# coding: utf-8

from __future__ import division, print_function

"""
Analytic transformations to action-angle coordinates.
"""


# Third-party
import numpy as np
from astropy.constants import G
import astropy.coordinates as coord
import astropy.units as u

# Project
from ..potential import (Hamiltonian, PotentialBase,
                         IsochronePotential, HarmonicOscillatorPotential)
from ..util import atleast_2d

__all__ = ['isochrone_to_aa', 'harmonic_oscillator_to_aa']

def isochrone_to_aa(w, potential):
    """
    Transform the input cartesian position and velocity to action-angle
    coordinates in the Isochrone potential. See Section 3.5.2 in
    Binney & Tremaine (2008), and be aware of the errata entry for
    Eq. 3.225.

    This transformation is analytic and can be used as a "toy potential"
    in the Sanders & Binney (2014) formalism for computing action-angle
    coordinates in any potential.

    .. note::

        This function is included as a method of the
        :class:`~gala.potential.IsochronePotential` and it is recommended
        to call :meth:`~gala.potential.IsochronePotential.phase_space()`
        instead.

    Parameters
    ----------
    w : :class:`gala.dynamics.PhaseSpacePosition`, :class:`gala.dynamics.Orbit`
    potential : :class:`gala.potential.IsochronePotential`, dict
        An instance of the potential to use for computing the transformation
        to angle-action coordinates. Or, a dictionary of parameters used to
        define an :class:`gala.potential.IsochronePotential` instance.

    Returns
    -------
    actions : :class:`numpy.ndarray`
        An array of actions computed from the input positions and velocities.
    angles : :class:`numpy.ndarray`
        An array of angles computed from the input positions and velocities.
    freqs : :class:`numpy.ndarray`
        An array of frequencies computed from the input positions and velocities.
    """

    if not isinstance(potential, PotentialBase):
        potential = IsochronePotential(**potential)

    usys = potential.units
    GM = (G*potential.parameters['m']).decompose(usys).value
    b = potential.parameters['b'].decompose(usys).value
    E = w.energy(Hamiltonian(potential)).decompose(usys).value
    E = np.squeeze(E)

    if np.any(E > 0.):
        raise ValueError("Unbound particle. (E = {})".format(E))

    # convert position, velocity to spherical polar coordinates
    w_sph = w.represent_as(coord.PhysicsSphericalRepresentation)
    r,phi,theta = map(np.squeeze, [w_sph.r.decompose(usys).value,
                                   w_sph.phi.radian,
                                   w_sph.theta.radian])

    ang_unit = u.radian/usys['time']
    vr,phi_dot,theta_dot = map(np.squeeze, [w_sph.radial_velocity.decompose(usys).value,
                                            w_sph.pm_phi.to(ang_unit).value,
                                            w_sph.pm_theta.to(ang_unit).value])
    vphi = r*np.sin(theta) * phi_dot
    vtheta = r*theta_dot

    # ----------------------------
    # Compute the actions
    # ----------------------------

    L_vec = np.squeeze(w.angular_momentum().decompose(usys).value)
    Lz = L_vec[2]
    L = np.linalg.norm(L_vec, axis=0)

    # Radial action
    Jr = GM / np.sqrt(-2*E) - 0.5*(L + np.sqrt(L*L + 4*GM*b))

    # compute the three action variables
    actions = np.array([Jr, Lz, L - np.abs(Lz)]) # Jr, Jphi, Jtheta

    # ----------------------------
    # Angles
    # ----------------------------
    c = GM / (-2*E) - b
    e = np.sqrt(1 - L*L*(1 + b/c) / GM / c)

    # Compute theta_r using eta
    tmp1 = r*vr / np.sqrt(-2.*E)
    tmp2 = b + c - np.sqrt(b*b + r*r)
    eta = np.arctan2(tmp1,tmp2)
    thetar = eta - e*c*np.sin(eta) / (c + b)  # same as theta3

    # Compute theta_z
    psi = np.arctan2(np.cos(theta), -np.sin(theta)*r*vtheta/L)
    psi[np.abs(vtheta) <= 1e-10] = np.pi/2.  # blows up for small vtheta

    omega_th = 0.5 * (1 + L/np.sqrt(L*L + 4*GM*b))

    a = np.sqrt((1+e) / (1-e))
    ap = np.sqrt((1 + e + 2*b/c) / (1 - e + 2*b/c))

    def F(x, y):
        z = np.zeros_like(x)

        ix = y>np.pi/2.
        z[ix] = np.pi/2. - np.arctan(np.tan(np.pi/2.-0.5*y[ix])/x[ix])

        ix = y<-np.pi/2.
        z[ix] = -np.pi/2. + np.arctan(np.tan(np.pi/2.+0.5*y[ix])/x[ix])

        ix = (y<=np.pi/2) & (y>=-np.pi/2)
        z[ix] = np.arctan(x[ix]*np.tan(0.5*y[ix]))
        return z

    A = omega_th*thetar - F(a,eta) - F(ap,eta)/np.sqrt(1 + 4*GM*b/L/L)
    thetaz = psi + A

    LR = Lz/L
    sinu = (LR/np.sqrt(1.-LR*LR)/np.tan(theta))
    uu = np.arcsin(sinu)

    uu[sinu > 1.] = np.pi/2.
    uu[sinu < -1.] = -np.pi/2.
    uu[vtheta > 0.] = np.pi - uu[vtheta > 0.]

    thetap = phi - uu + np.sign(Lz)*thetaz
    angles = np.array([thetar, thetap, thetaz])
    angles = angles % (2*np.pi)

    # ----------------------------
    # Frequencies
    # ----------------------------
    freqs = np.zeros_like(actions)
    omega_r = GM**2 / (Jr + 0.5*(L + np.sqrt(L*L + 4*GM*b)))**3
    freqs[0] = omega_r
    freqs[1] = np.sign(actions[1]) * omega_th * omega_r
    freqs[2] = omega_th * omega_r

    a_unit = (1*usys['angular momentum']).decompose(usys).unit
    f_unit = (1*usys['frequency']).decompose(usys).unit
    return actions*a_unit, angles*u.radian, freqs*f_unit

def isochrone_to_xv(actions, angles, potential):
    """
    Transform the input actions and angles to ordinary phase space (position
    and velocity) in cartesian coordinates. See Section 3.5.2 in
    Binney & Tremaine (2008), and be aware of the errata entry for
    Eq. 3.225.

    .. note::

        This function is included as a method of the :class:`~gala.potential.IsochronePotential`
        and it is recommended to call :meth:`~gala.potential.IsochronePotential.action_angle()`
        instead.

    Parameters
    ----------
    actions : array_like
        Action variables. Must have shape ``(3,N)`` or ``(3,)``.
    angles : array_like
        Angle variables. Must have shape ``(3,N)`` or ``(3,)``.
        Should be in radians.
    potential : :class:`gala.potential.IsochronePotential`
        An instance of the potential to use for computing the transformation
        to angle-action coordinates.

    Returns
    -------
    x : :class:`numpy.ndarray`
        An array of cartesian positions computed from the input
        angles and actions.
    v : :class:`numpy.ndarray`
        An array of cartesian velocities computed from the input
        angles and actions.
    """

    raise NotImplementedError("Implementation not supported until working with "
                              "angle-action variables has a better API.")

    # actions = atleast_2d(actions,insert_axis=1).copy()
    # angles = atleast_2d(angles,insert_axis=1).copy()

    # usys = potential.units
    # GM = (G*potential.parameters['m']).decompose(usys).value
    # b = potential.parameters['b'].decompose(usys).value

    # # actions
    # Jr = actions[0]
    # Lz = actions[1]
    # L = actions[2] + np.abs(Lz)

    # # angles
    # theta_r,theta_phi,theta_theta = angles

    # # get longitude of ascending node
    # theta_1 = theta_phi - np.sign(Lz)*theta_theta
    # Omega = theta_1

    # # Ly = -np.cos(Omega) * np.sqrt(L**2 - Lz**2)
    # # Lx = np.sqrt(L**2 - Ly**2 - Lz**2)
    # cosi = Lz/L
    # sini = np.sqrt(1 - cosi**2)

    # # Hamiltonian (energy)
    # H = -2. * GM**2 / (2.*Jr + L + np.sqrt(4.*b*GM + L**2))**2

    # if np.any(H > 0.):
    #     raise ValueError("Unbound particle. (E = {})".format(H))

    # # Eq. 3.240
    # c = -GM / (2.*H) - b
    # e = np.sqrt(1 - L*L*(1 + b/c) / GM / c)

    # # solve for eta
    # theta_3 = theta_r
    # eta_func = lambda x: x - e*c/(b+c)*np.sin(x) - theta_3
    # eta_func_prime = lambda x: 1 - e*c/(b+c)*np.cos(x)

    # # use newton's method to find roots
    # niter = 100
    # eta = np.ones_like(theta_3)*np.pi/2.
    # for i in range(niter):
    #     eta -= eta_func(eta)/eta_func_prime(eta)

    # # TODO: when to do this???
    # eta -= 2*np.pi

    # r = c*np.sqrt((1-e*np.cos(eta)) * (1-e*np.cos(eta) + 2*b/c))
    # vr = np.sqrt(GM/(b+c))*(c*e*np.sin(eta))/r

    # theta_2 = theta_theta
    # Omega_23 = 0.5*(1 + L / np.sqrt(L**2 + 4*GM*b))

    # a = np.sqrt((1+e) / (1-e))
    # ap = np.sqrt((1 + e + 2*b/c) / (1 - e + 2*b/c))

    # def F(x, y):
    #     z = np.zeros_like(x)

    #     ix = y>np.pi/2.
    #     z[ix] = np.pi/2. - np.arctan(np.tan(np.pi/2.-0.5*y[ix])/x[ix])

    #     ix = y<-np.pi/2.
    #     z[ix] = -np.pi/2. + np.arctan(np.tan(np.pi/2.+0.5*y[ix])/x[ix])

    #     ix = (y<=np.pi/2) & (y>=-np.pi/2)
    #     z[ix] = np.arctan(x[ix]*np.tan(0.5*y[ix]))
    #     return z

    # theta_2[Lz < 0] -= 2*np.pi
    # theta_3 -= 2*np.pi
    # A = Omega_23*theta_3 - F(a,eta) - F(ap,eta)/np.sqrt(1 + 4*GM*b/L/L)
    # psi = theta_2 - A

    # # theta
    # theta = np.arccos(np.sin(psi)*sini)
    # vtheta = L*sini*np.cos(psi)/np.cos(theta)
    # vtheta = -L*sini*np.cos(psi)/np.sin(theta)/r
    # vphi = Lz / (r*np.sin(theta))

    # d_phi = vphi / (r*np.sin(theta))
    # d_theta = vtheta / r

    # # phi
    # sinu = np.sin(psi)*cosi/np.sin(theta)

    # uu = np.arcsin(sinu)
    # uu[sinu > 1.] = np.pi/2.
    # uu[sinu < -1.] = -np.pi/2.
    # uu[vtheta > 0.] = np.pi - uu[vtheta > 0.]

    # sinu = cosi/sini * np.cos(theta)/np.sin(theta)
    # phi = (uu + Omega) % (2*np.pi)

    # # We now need to convert from spherical polar coord to cart. coord.
    # pos = coord.PhysicsSphericalRepresentation(r=r*u.dimensionless_unscaled,
    #                                            phi=phi*u.rad, theta=theta*u.rad)
    # pos = pos.represent_as(coord.CartesianRepresentation)
    # x = pos.xyz.value

    # vel = coord.PhysicsSphericalDifferential(d_phi=d_phi, d_theta=d_theta, d_r=vr)
    # v = vel.represent_as(coord.CartesianDifferential, base=pos).d_xyz.value

    # return x, v

def harmonic_oscillator_to_aa(w, potential):
    """
    Transform the input cartesian position and velocity to action-angle
    coordinates for the Harmonic Oscillator potential.

    This transformation is analytic and can be used as a "toy potential"
    in the Sanders & Binney (2014) formalism for computing action-angle
    coordinates in any potential.

    .. note::

        This function is included as a method of the
        :class:`~gala.potential.HarmonicOscillatorPotential`
        and it is recommended to call
        :meth:`~gala.potential.HarmonicOscillatorPotential.action_angle()` instead.

    Parameters
    ----------
    w : :class:`gala.dynamics.PhaseSpacePosition`, :class:`gala.dynamics.Orbit`
    potential : Potential
    """

    usys = potential.units
    if usys is not None:
        x = w.xyz.decompose(usys).value
        v = w.v_xyz.decompose(usys).value
    else:
        x = w.xyz.value
        v = w.v_xyz.value
    _new_omega_shape = (3,) + tuple([1]*(len(x.shape)-1))

    # compute actions -- just energy (hamiltonian) over frequency
    if usys is None:
        usys = []

    try:
        omega = potential.parameters['omega'].reshape(_new_omega_shape).decompose(usys).value
    except AttributeError: # not a Quantity
        omega = potential.parameters['omega'].reshape(_new_omega_shape)

    action = (v**2 + (omega*x)**2)/(2.*omega)

    angle = np.arctan(-v / omega / x)
    angle[x == 0] = -np.sign(v[x == 0])*np.pi/2.
    angle[x < 0] += np.pi

    freq = potential.parameters['omega'].decompose(usys).value

    if usys is not None and usys:
        a_unit = (1*usys['angular momentum']).decompose(usys).unit
        f_unit = (1*usys['frequency']).decompose(usys).unit
        return action*a_unit, (angle % (2.*np.pi))*u.radian, freq*f_unit
    else:
        return action*u.one, (angle % (2.*np.pi))*u.one, freq*u.one

def harmonic_oscillator_to_xv(actions, angles, potential):
    """
    Transform the input action-angle coordinates to cartesian
    position and velocity for the Harmonic Oscillator potential.

    .. note::

        This function is included as a method of the
        :class:`~gala.potential.HarmonicOscillatorPotential`
        and it is recommended to call
        :meth:`~gala.potential.HarmonicOscillatorPotential.phase_space()` instead.

    Parameters
    ----------
    actions : array_like
    angles : array_like
    potential : Potential
    """
    raise NotImplementedError("Implementation not supported until working with "
                              "angle-action variables has a better API.")

    # TODO: bug in below...
    # omega = potential.parameters['omega'].decompose(potential.units).value
    # x = np.sqrt(2*actions/omega[None]) * np.sin(angles)
    # v = np.sqrt(2*actions*omega[None]) * np.cos(angles)

    # return x,v



# coding: utf-8
# cython: boundscheck=False
# cython: nonecheck=False
# cython: cdivision=True
# cython: wraparound=False
# cython: profile=False

from __future__ import division, print_function

# Standard library
from collections import OrderedDict

# Third-party
from astropy.utils.misc import isiterable
import astropy.units as u
import numpy as np
cimport numpy as np
np.import_array()

# Project
from ..cframe import CFrameBase
from ..cframe cimport CFrameWrapper
from ....units import dimensionless, DimensionlessUnitSystem

cdef extern from "src/funcdefs.h":
    ctypedef double (*energyfunc)(double t, double *pars, double *q, int n_dim) nogil
    ctypedef void (*gradientfunc)(double t, double *pars, double *q, int n_dim, double *grad) nogil
    ctypedef void (*hessianfunc)(double t, double *pars, double *q, int n_dim, double *hess) nogil

cdef extern from "frame/src/cframe.h":
    ctypedef struct CFrame:
        energyfunc energy
        gradientfunc gradient
        hessianfunc hessian

        int n_params
        double *parameters;

cdef extern from "frame/builtin/builtin_frames.h":
    double static_frame_hamiltonian(double t, double *pars, double *qp, int n_dim) nogil
    void static_frame_gradient(double t, double *pars, double *qp, int n_dim, double *grad) nogil
    void static_frame_hessian(double t, double *pars, double *qp, int n_dim, double *hess) nogil

    double constant_rotating_frame_hamiltonian_2d(double t, double *pars, double *qp, int n_dim) nogil
    void constant_rotating_frame_gradient_2d(double t, double *pars, double *qp, int n_dim, double *grad) nogil
    void constant_rotating_frame_hessian_2d(double t, double *pars, double *qp, int n_dim, double *hess) nogil

    double constant_rotating_frame_hamiltonian_3d(double t, double *pars, double *qp, int n_dim) nogil
    void constant_rotating_frame_gradient_3d(double t, double *pars, double *qp, int n_dim, double *grad) nogil
    void constant_rotating_frame_hessian_3d(double t, double *pars, double *qp, int n_dim, double *hess) nogil

__all__ = ['StaticFrame', 'ConstantRotatingFrame']

cdef class StaticFrameWrapper(CFrameWrapper):

    def __init__(self):
        cdef CFrame cf

        cf.energy = <energyfunc>(static_frame_hamiltonian)
        cf.gradient = <gradientfunc>(static_frame_gradient)
        cf.hessian = <hessianfunc>(static_frame_hessian)
        cf.n_params = 0
        cf.parameters = NULL

        self.cframe = cf

class StaticFrame(CFrameBase):
    """
    Represents a static intertial reference frame.

    Parameters
    ----------
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, units=None):
        # ndim = None means it can support any number of ndim
        super(StaticFrame, self).__init__(StaticFrameWrapper, dict(), units, ndim=None)

# ---

cdef class ConstantRotatingFrameWrapper2D(CFrameWrapper):

    def __init__(self, double Omega):
        cdef:
            CFrame cf

        self._params = np.array([Omega], dtype=np.float64)

        cf.energy = <energyfunc>(constant_rotating_frame_hamiltonian_2d)
        cf.gradient = <gradientfunc>(constant_rotating_frame_gradient_2d)
        cf.hessian = <hessianfunc>(constant_rotating_frame_hessian_2d)
        cf.n_params = 1
        cf.parameters = &(self._params[0])

        self.cframe = cf

cdef class ConstantRotatingFrameWrapper3D(CFrameWrapper):

    def __init__(self, double Omega_x, double Omega_y, double Omega_z):
        cdef:
            CFrame cf

        self._params = np.array([Omega_x, Omega_y, Omega_z], dtype=np.float64)

        cf.energy = <energyfunc>(constant_rotating_frame_hamiltonian_3d)
        cf.gradient = <gradientfunc>(constant_rotating_frame_gradient_3d)
        cf.hessian = <hessianfunc>(constant_rotating_frame_hessian_3d)
        cf.n_params = 3
        cf.parameters = &(self._params[0])

        self.cframe = cf

class ConstantRotatingFrame(CFrameBase):
    """
    Represents a constantly rotating reference frame.

    The reference frame rotates with constant angular velocity set by the
    magnitude of the vector parameter ``Omega`` around the axis defined by
    the unit vector computed from the input frequency vector.

    Parameters
    ----------
    Omega : :class:`~astropy.units.Quantity`
        The frequency vector, which specifies the axis of rotation and the
        angular velocity of the frame.
    units : `~gala.units.UnitSystem` (optional)
        Set of non-reducable units that specify (at minimum) the
        length, mass, time, and angle units.

    """
    def __init__(self, Omega, units=None):

        if units is None:
            units = dimensionless

        Omega = np.atleast_1d(Omega)

        if Omega.shape == (1,):
            # assumes ndim=2, must be associated with a 2D potential
            Omega = np.squeeze(Omega)
            ndim = 2
            ConstantRotatingFrameWrapper = ConstantRotatingFrameWrapper2D

        elif Omega.shape == (3,):
            # assumes ndim=3, must be associated with a 3D potential
            ndim = 3
            ConstantRotatingFrameWrapper = ConstantRotatingFrameWrapper3D

        else:
            raise ValueError("Invalid input for rotation vector Omega.")

        pars = OrderedDict()
        ptypes = OrderedDict()
        pars['Omega'] = Omega
        ptypes['Omega'] = 'frequency'

        super(ConstantRotatingFrame, self).__init__(ConstantRotatingFrameWrapper,
                                                    pars, units, ndim=ndim)

        if self.parameters['Omega'].unit != u.one and isinstance(units, DimensionlessUnitSystem):
            raise ValueError("If frequency vector has units, you must pass in a unit system.")
